library(ggplot2)
library(gridExtra)

# function to convert rgb to hex
rgb2hex <-
function(col) {
    rgb(col[1],col[2],col[3],  maxColorValue=255)
}

# information about T. chumash individuals, including colour (coded in hex)
# (provided by R. Villoutreix)
inds.info<-read.csv("2015Timema_sp_complete.csv", header=T)

# 10 SNPs from scaffold 128 (LG8) most associated with latRG in T. chummash
geno<-read.table("10snps_lg8scaf128_latRG.dsv", header=T)
pheno<-scan("/fastdata-sharc/bo1mdlmc/GWAS_Tchumash/latRG/Tchumash_GR8.06.latRG.pheno")
names(pheno)<-colnames(geno)[-c(1:3)]

# melanic allele dosage (sum of allele dosages for all SNPs)
meldosage<-apply(geno[,-c(1:3)], 2, sum)

# data frame with melanic allele dosage and phenotype scores
mel.df<-data.frame(meldosage,pheno)

# colour of bugs
colours<-as.character(inds.info$lat.s)
names(colours)<-paste("X",inds.info$id, sep="")
colours<-colours[names(pheno)]

# plot melanic allele dosage vs latRG and colour dots by real bug colour
fit1 <-lm(meldosage~pheno, data= mel.df)
 
meldos.latrg<-
ggplot(fit1$model, aes_string(x = names(fit1$model)[1], y = names(fit1$model)[2])) +
     geom_point(show.legend=F, color=colours) +
     stat_smooth(method = "lm", col = "blue") +
	     xlab("Melanic allele dosage") +
	     ylab("latRG") +
	     xlim(0, 20) +
	     theme_classic() +
	     geom_label(aes(x = 16, y = 2.5), hjust = 0,
         label = paste("Adj R2 = ",signif(summary(fit1)$adj.r.squared, 5),
	                      "\nIntercept =",signif(fit1$coef[[1]],5 ),
                          "\nSlope =",signif(fit1$coef[[2]], 5),
	                      "\nP =",signif(summary(fit1)$coef[2,4], 5)))


# mean color for each category
phenosi<-list()
for (i in 0:20){
	idsi<-names(meldosage[meldosage==i])
	if (length(idsi)>0){
		phenosi[[i+1]]<-as.numeric(pheno[idsi])
	}
	else{
		phenosi[[i+1]]<-NA
	}
}


# discretize to integers to be used in boxplots
meldosage.discrete<-round(apply(geno[,-c(1:3)], 2, sum))
mel.df2<-data.frame(meldosage.discrete, pheno)

# mean colour for each discrete category (0-20)
mean.colours<-vector()
for (md in sort(unique(meldosage.discrete))){
	ids<-rownames(mel.df2[mel.df2$meldosage.discrete==md,])
	ids.cols<-colours[ids]
	mean.col<-rgb2hex(as.numeric(apply(col2rgb(ids.cols), 1, mean)))
	mean.colours<-c(mean.colours, mean.col)
}

meldos.latrg.boxplots<-
ggplot(mel.df2, aes(x=as.factor(meldosage.discrete), y=pheno, colour=pheno)) +
	geom_boxplot(show.legend=F, fill=mean.colours) +
	xlab("Melanic allele dosage") + 
	ylab("latRG") +
    scale_x_discrete(limits=c("0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20")) +
	theme_classic()
	


pdf("melanic_dosage_latRG_lm_se_ley.pdf", width=8.27, height=11.69);
	grid.arrange(meldos.latrg, meldos.latrg.boxplots, nrow=2)
dev.off()

