#!/usr/local/extras/Genomics/bin/Rscript --vanilla

# (c) 2020 - Victor Soria-Carrasco
# This is a wrapper to run MAPIT analyses (in combination with runMAPIT.R)
# See https://github.com/lorinanthony/MAPIT and Crawford (2017) 
# (https://doi.org/10.1371/journal.pgen.1006869) for more information

library(getopt)
library(tools)

# get arguments C-style
spec<-matrix(c(
  'mapitrun',      'm',  1, "character",
  'threshold',     't',  1, "double",
  'help',          'h',  0, "logical"
), byrow=T, ncol=4)
opt<-getopt(spec)

cat ("\n")
if (is.null(opt$help) & is.null(opt$mapitrun)) cat("ERROR: R data file with output of MAPIT run not specified\n\n")

# show usage as help
if (!is.null(opt$help) || is.null(opt$mapitrun)) {
	cat(getopt(spec, usage=TRUE));
	cat("\n")
	q(status=1);
}

# use default values if none is given
if (is.null(opt$threshold)) opt$threads<-2

cat ("\nArguments:\n")
cat ("-------------------------------------------------------------------------------\n")
cat ("  mapitrun:      ", opt$mapitrun,"\n")
cat ("  threshold:     ", opt$threshold,"\n")
cat ("-------------------------------------------------------------------------------\n\n")

### MAPIT Rdata file with MAPIT results from runMAPIT.R
mapitrunfile<-opt$mapitrun

### Threshold for significant SNPs according to their marginal epistatic effects
### to run a simple exhaustive search between them
threshold<-opt$threshold

### output directory
outdir<-dirname(mapitrunfile)

# echo cmds
options(echo=TRUE)

# Show time
Sys.time()

### load MAPIT results

load(mapitrunfile)

# mapit is a list expected to have the following fields
# 	mapit[["snp"]] # name of snps
# 	mapit[["ind"]] # name of individuals
# 	mapit[["X"]]   # genotypes
# 	mapit[["y"]] # phenotypes
# 	mapit[["C"]] # covariates
# 	mapit[["W"]] # covariates population structure and environment
# 	mapit[["method"]] # method
# 	mapit[["threshold"]] # threshold for recalculating p-values using Davies method
# 	mapit[["runtime"]] # running time


X<-mapit$X # genotypes
y<-mapit$y # phenotypes
pvalues<-mapit$pvalues # margianl epistatic effect p-values
snps<-mapit$snp # snps ids
method<-mapit$method # method used in MAPIT run

# Use Bonferroni correction as default threshold if none is given
if (is.null(threshold)) threshold<-0.05/length(pvalues)

# Output of pairwise p-values
outpwise<-paste(outdir, "/", file_path_sans_ext(basename(mapitrunfile)),".MAPIT", method, ".pwise.thr",threshold,".dsv", sep="")

### Run an Informed Exhaustive Search ###
# NOTE: Now we may take only the significant SNPs according to their marginal epistatic effects
# and run a simple exhaustive search between them

pvalues<-pvalues[!is.na(pvalues)] # Remove NAs
v<-pvalues[pvalues<=threshold] # Call only marginally significant SNPs
rownames(X)<-snps
names(v)<-snps[pvalues<=threshold]
pwise<-vector()
for (k in 1:length(v)){
	fit<-vector();
	m<-1:length(v)
	for(w in m[-k]){
		pt<-lm(y~X[names(v)[k],]:X[names(v)[w],])
		fit[w]<-coefficients(summary(pt))[8]
		names(fit)[w]<-paste(names(v)[k],names(v)[w],sep = "-")
	}
	pwise<-c(pwise,fit)
}

### Sort and keep only unique pairs ###
vc.pwise<-sort(pwise[!is.na(pwise)]) # Get rid of the NAs and sort
vc.pwise<-vc.pwise[seq(1,length(vc.pwise),2)] # Only keep the unique pairs
vc.pwise<-sort(vc.pwise) # Sort the pairs in order of significance

# Save results to table
out<-data.frame(pair=names(vc.pwise), pvalue=as.numeric(vc.pwise))
write.table(out, file=outpwise, sep="\t", row.names=F, col.names=T, quote=F)

Sys.time()

######################################################################################
######################################################################################
