#!/bin/bash

# (c) 2020 - Victor Soria-Carrasco
# victor.soria.carrasco@gmail.com
# Last modified: 23/08/2017 16:01:38

# Description:
# Given an input directory with raw fastq files (gzip and bzip2 compression allowed)
# and a directory with barcodes files (they must have the same names than the fastq files),
# it will parse the barcodes of the sequences


VERSION='2.0-2017.08.23'
PARSER="$(dirname $0)/parse_barcodes768_vsc.pl"
PARALLEL='parallel'
COMP='lbzip2 -n '

# number of cores for compression/decompression during splitting
# use all available cores by default
if [[ -z $NTHREADS ]];
then
	export NTHREADS=$(cat /proc/cpuinfo | grep -ch processor)
fi

# Default values for optional variables
EXTRA=''
EMAIL=''
QUEUE='iceberg'
HRS=24
MEM=4
NPAR=1
NTHR=1
NTHRSUM=4
CLEAN=0
function author {
	echo
	echo "#########################################"
	echo "  $(basename $0)"
	echo "  version $VERSION"
	echo "  (c) Victor Soria-Carrasco"
	echo "  victor.soria.carrasco@gmail.com"
	echo "#########################################"
	echo
}

function usage {
	echo
	echo "Usage:"
	echo "  $(basename $0)"
	echo "      -i <input directory> => Folder with fastq files (.gz and .bz2 accepted)"
	echo "      -p <input directory> => Folder with paired-end fastq files (.gz and .bz2 accepted) (optional)"
	echo "      -b <barcodes directory> => Folder with barcodes files (.dsv or .csv)"
	echo "      -o <output directory> => Output folder to save parsed files"
	echo "      -s <number of tasks per library> => Parallelize by splitting each library in a number of tasks (optional: default=1)"
	echo "      -n <number of threads> => Number of threads used for compression (optional: default=1)"
	echo "      -z <number of threads> => Number of threads used for compression during summary (optional: default=4)"
	echo "      -t <allocated time> => Allocated time (in hours) for each analysis (optional: default=24)"
	echo "      -m <allocated memory> => Allocated memory (in gigabytes) for each analysis (optional: default=4)"
	echo "      -q <queue> => SGE queue: iceberg | popgenom | molecol (optional: default=iceberg)"
	echo "      -e <email> => Notification email address (default=none)"
	echo "      -c => Clean intermediate files (default=no)"
	echo "      -h => show this help"
	echo ""
	echo "  Example:"
	echo "      $(basename $0) -i raw_reads -o parsed_reads -q popgenom"
	echo ""
	echo ""
	exit 0
}

author

if [ "$#" -ge "6" ]; # min 6 args: 2 for -i <input directory>, 2 for -b <barcodes directory>, 2 for -o <output directory>
then 
	while [ $# -gt 0 ]; do
		case "$1" in
			-h|-help) usage
					  ;;
			-i)	shift
				INDIR=$(readlink -f $1)
				;;
			-p)	shift
				INDIRP=$(readlink -f $1)
				;;
			-b)	shift
				BARCODESDIR=$(readlink -f $1)
				;;
			-o)	shift
				OUTDIR=$(readlink -f $1)
				;;
			-s)	shift
				NPAR=$1
				;;
			-n)	shift
				NTHR=$1
				;;
			-z)	shift
				NTHRSUM=$1
				;;
			-t)	shift
				HRS=$1
				;;
			-m)	shift
				MEM=$1
				;;
			-q)	shift
				QUEUE=$1
				;;
			-e)	shift
				EMAIL=$1
				;;
			-c)	CLEAN=1
				;;
			*)	echo 
				echo "ERROR - Invalid option: $1"
				echo
				usage
				;;
		esac
		shift
	done
else
	usage
fi

# check input directory
if [[ -z $INDIR ]]; then
	echo
	echo "ERROR: directory of reads specified with -i does not exist"
	echo
	exit
fi

# check paired end files
PAIREND=0
if [[ ! -z $INDIRP ]]; then
	PAIREND=1
	if [[ ! -d $INDIRP ]];
	then
		echo
		echo "ERROR: directory of paired end specifiec with -p does not exist"
		echo
		exit
	fi
fi

# check barcodes
FQS=$(find $INDIR -name "*.fq*" -or -name "*.fastq*")
for f in $FQS;
do
	f=$(basename $f)
	bc="$BARCODESDIR/${f%%.*}.barcodes"
	if [[ ! -e $bc.dsv && ! -e $bc.csv ]];
	then
		echo
		echo "ERROR: Barcodes files for $INDIR/$f not found in $BARCODESDIR"
		echo
		exit
	fi
done

# check number of read1 and read2 files matches
N=$(find $INDIR -maxdepth 1 -name "*.fq*" -or -name "*.fastq*" | wc -l | cut -f1 -d" ")
if [[ $PAIREND == 1 ]];
then
	NP=$(find $INDIRP -maxdepth 1 -name "*.fq*" -or -name "*.fastq*" | wc -l | cut -f1 -d" ")
	if [[ "$N" -ne "$NP" ]]; then
		echo
		echo $INDIRP
		echo "ERROR: Different number of files for reads 1 ($N) and reads2 ($NP)"
		echo
		exit
	fi
fi

# Split files for parallel processing
# =============================================================================
if [[ $NPAR -gt 1 ]];
then
	echo "Splitting each input file into $NPAR parts for parallel processing..."
	echo

	# function to split fastq files
	# ............................................................................
	function fqsplit(){
		FQ=$1 # input fastq file
		OUTDIR=$2 # output directory
		NPAR=$3
		NTHR=$4

		NTOTALLINES=$($COMP $NTHREADS -dck $FQ | wc -l)
		NLINES=$((($NTOTALLINES + $NPAR-1) / $NPAR)) # round up to ensure exactly $NPAR files are generated
		NLINES=$((4 * (($NLINES + 3) / 4))) # round up to nearest multiple of 4

		$COMP $NTHR -dck $FQ | \
		split -l $NLINES -d -a 2 --additional-suffix ".fq" --filter 'lbzip2 -n '$NTHR' > $FILE.bz2' - "$OUTDIR/$(basename ${FQ%%.*})-"
	}
	export -f fqsplit
	# ............................................................................

	N=$(($N*$NPAR)) # number of tasks

	# get fq files into arrays and create directories
	FQS=($(find $INDIR -name "*.fq*" -or -name "*.fastq*" | sort -V ))
	if [[ $PAIREND == 1 ]];
	then
		FQPS=($(find $INDIRP -name "*.fq*" -or -name "*.fastq*" | sort -V ))
		mkdir -p $OUTDIR/reads1-split
		mkdir -p $OUTDIR/reads2-split
		INDIR=$OUTDIR/reads1-split
		INDIRP=$OUTDIR/reads2-split
	else
		mkdir -p $OUTDIR/reads-split
		INDIR=$OUTDIR/reads-split
	fi
	mkdir -p $OUTDIR/barcodes-split

	if [[ ${FQS[0]##.*} == "gz" ]];	then
		COMP='pigz -p '
	fi
	export COMP
	
	# split files
	if [[ $PAIREND == 1 ]];
	then
		ALLFQS=( "${FQS[@]}" "${FQPS[@]}" )
		OUTDIRS=($(printf "$OUTDIR/reads1-split%.0s " $(seq 1 ${#FQS[@]})))
		OUTDIRS+=($(printf "$OUTDIR/reads2-split%.0s " $(seq 1 ${#FQPS[@]})))
		NTHRJOB=$((($NTHREADS + ${#ALLFQS[@]} - 1)/${#ALLFQS[@]})) # round up number of threads per file to split to avoid zero
		$PARALLEL -j $NTHREADS --link fqsplit ::: ${ALLFQS[@]} ::: ${OUTDIRS[@]} ::: $NPAR ::: $NTHRJOB
	else
		NTHRJOB=$((($NTHREADS + ${#FQS[@]} - 1)/${#FQS[@]})) # round up number of threads per file to split to avoid zero
		$PARALLEL -j $NTHREADS fqsplit ::: ${FQS[@]} ::: "$OUTDIR/reads-split" ::: $NPAR ::: $NTHRJOB
	fi

	# copy barcodes too
	for i in "${!FQS[@]}";
	do
		BARCODES=$(find $BARCODESDIR -name "$(basename ${FQS[i]%%.*}).barcodes.*")
		seq 0 $(($NPAR-1)) | xargs -I {} sh -c 'cp '$BARCODES' '$OUTDIR'/barcodes-split/'$(basename ${BARCODES%%.*})'-$(printf "%02d" {}).barcodes.'${BARCODES##*.}''
	done

	# set new barcodes directory	
	BARCODESDIR="$OUTDIR/barcodes-split/"
fi
# =============================================================================

# Set temporary directory
if [[ $QUEUE == "popgenom" || $QUEUE == "molecol" ]];
then
	TMPDIR=/local/tmp
else
	TMPDIR=/scratch/tmp
fi

# Set script and log filenames
TIMESTAMP=$(date +%Y%m%d-%H%M%S)
SMSJOB="$OUTDIR/parse_barcodes.$TIMESTAMP.smsjob.sh"
LOG="$OUTDIR/parse_barcodes.$TIMESTAMP.smsjob.log"
SMSJOBSUM="$OUTDIR/parse_barcodes.$TIMESTAMP.smsjobsum.sh"
LOGSUM="$OUTDIR/parse_barcodes.$TIMESTAMP.smsjobsum.log"

# Submission script
# =============================================================================
mkdir -p $OUTDIR
echo '#!/bin/bash' > $SMSJOB

# SGE OPTIONS
# -----------------------------------------
echo '#$ -l h_rt='$HRS':00:00' >> $SMSJOB
echo '#$ -l rmem='$MEM'G' >> $SMSJOB
echo '#$ -l mem='$MEM'G' >> $SMSJOB

if [ $NTHR -gt 1 ];
then
	echo '#$ -pe openmp '$NTHR >> $SMSJOB
fi

if [ ! -z "$EMAIL" ];
then
	echo '#$ -m bea' >> $SMSJOB
	echo '#$ -M '$EMAIL >> $SMSJOB
fi
if [[ $QUEUE == 'popgenom' || $QUEUE == 'molecol' ]];
then
	echo '#$ -P '$QUEUE >> $SMSJOB
	echo '#$ -q '$QUEUE'.q' >> $SMSJOB
fi
echo '#$ -t 1-'$N >> $SMSJOB
echo '#$ -j y' >> $SMSJOB
echo '#$ -o '$LOG >> $SMSJOB
echo '#$ -V' >> $SMSJOB
echo '#$ -N parse_barcodes.'$TIMESTAMP >> $SMSJOB
# -----------------------------------------

cat >> $SMSJOB <<EOF


INDIR=$INDIR
BARCODESDIR=$BARCODESDIR
OUTDIR=$OUTDIR
FQFILES=(\$(find \$INDIR -name "*.fq*" -or -name "*.fastq*" | sort -V))
INDEX=\$((SGE_TASK_ID-1))
FQ=\${FQFILES[\$INDEX]}
EOF

if [ $PAIREND == 1 ]; then
cat >> $SMSJOB <<EOF

INDIRP=$INDIRP
FQFILESP=(\$(find \$INDIRP -name "*.fq*" -or -name "*.fastq*" | sort -V ))
FQP=\${FQFILESP[\$INDEX]}
EOF
fi


cat >> $SMSJOB <<EOF

FQB=\$(basename \$FQ)
LOG="\$OUTDIR/"\${FQB%%.*}".log"

# temporary directory for this job
export TMPDIR="$TMPDIR"
if [[ ! -d $TMPDIR ]];
then
	mkdir -p \$TMPDIR
fi
TIMESTAMP=\$(date +%Y%m%d-%H%M%S)
RUNTMPDIR=\$(mktemp --tmpdir -d \$USER-parse_barcodes_\$TIMESTAMP.XXXXXXXXXX)

hostname > \$LOG
uname -a >> \$LOG
date >> \$LOG
echo "---------------------------------------------------" >> \$LOG
echo >> \$LOG
echo "Using temporary directory \$RUNTMPDIR" >> \$LOG
echo >> \$LOG

TMPINDIR=\$RUNTMPDIR/in
TMPOUTDIR=\$RUNTMPDIR/out

# Create temporary local directories
if [ ! -e \$TMPINDIR ];
then
	mkdir -p \$TMPINDIR
fi

if [ ! -e \$TMPOUTDIR ];
then
	mkdir -p \$TMPOUTDIR
fi

# copy to local temporary directory
cp \$FQ \$TMPINDIR/
FQ=\$(basename \$FQ)

BARCODES="\$BARCODESDIR/"\${FQ%%.*}".barcodes.dsv"
if [ ! -e \$BARCODES ];
then
	BARCODES="\$BARCODESDIR/"\${FQ%%.*}".barcodes.csv"
fi
cp \$BARCODES \$TMPINDIR/
BARCODES=\$(basename \$BARCODES)

EOF

if [[ $PAIREND == 1 ]]; then
	echo "cp \$FQP \$TMPINDIR/"  >> $SMSJOB
	echo "FQP=\$(basename \$FQP)"  >> $SMSJOB
	echo "" >> $SMSJOB
	echo 'echo -e "Parsing $FQ file\n(paired with $FQP)\nusing $BARCODES barcodes..." >> $LOG' >> $SMSJOB 
else
	echo "" >> $SMSJOB
	echo 'echo -e "Parsing $FQ file\nusing $BARCODES barcodes..." >> $LOG' >> $SMSJOB 
fi

cat >> $SMSJOB <<EOF

echo >> \$LOG 
echo "CMD: " >> \$LOG
echo "$PARSER \\\" >> \$LOG
echo "-i \$FQ \\\" >> \$LOG
EOF
if [ $PAIREND == 1 ]; then
cat >> $SMSJOB <<EOF
echo "-p \$FQP \\\">> \$LOG
EOF
fi
cat >> $SMSJOB <<EOF
echo "-b \$BARCODES \\\">> \$LOG
echo "-n $NTHR \\\">> \$LOG
echo "-o \$OUTDIR" >> \$LOG
echo >> \$LOG
echo "---------------------------------------------------" >> \$LOG
echo >> \$LOG

cd \$TMPINDIR

$PARSER \\
-f \$FQ \\
EOF

if [ $PAIREND == 1 ]; then
	echo "-p \$FQP \\" >> $SMSJOB
fi

cat >> $SMSJOB <<EOF
-b \$BARCODES \\
-n $NTHR \\
-o \$TMPOUTDIR \\
>> \$LOG 2>&1
# -q \\

# Copy results back to output directory
# and check all files are good
EOF

if [ $PAIREND == 1 ];
then

cat >> $SMSJOB <<EOF
PARSED1=\$(echo -n \$FQ | perl -pe 's/\.(f(ast)?q).*/\_parsed_1\.\$1\.bz2/g')
MIDERRORS1=\$(echo -n \$FQ | perl -pe 's/\.(f(ast)?q).*/\_miderrors_1\.\$1\.bz2/g')
PARSED2=\$(echo -n \$FQP | perl -pe 's/\.(f(ast)?q).*/\_parsed_2\.\$1\.bz2/g')
MIDERRORS2=\$(echo -n \$FQP | perl -pe 's/\.(f(ast)?q).*/\_miderrors_2\.\$1\.bz2/g')

REPORT=\$(echo -n \$FQ | perl -pe 's/\.f(ast)?q.*/\_parsed\.report\.txt/g')

NTRY=0
while [[ ! -e \$TMPOUTDIR/reads1/\$PARSED1 || ! -e \$OUTDIR/reads1/\$PARSED1 || \
         ! -e \$TMPOUTDIR/reads2/\$PARSED2 || ! -e \$OUTDIR/reads2/\$PARSED2 || \
	     "\$(md5sum \$TMPOUTDIR/reads1/\$PARSED1 | awk '{print \$1}')" != "\$(md5sum \$OUTDIR/reads1/\$PARSED1 | awk '{print \$1}')" || \
		 "\$(md5sum \$TMPOUTDIR/reads2/\$PARSED2 | awk '{print \$1}')" != "\$(md5sum \$OUTDIR/reads2/\$PARSED2 | awk '{print \$1}')" ]] \
		 && [[ \$NTRY -lt 10 ]];
do
	sleep \$((30+\$RANDOM%90)) # wait 30-120 secs to try again
	echo "Transferring files to final destination \$OUTDIR" >> \$LOG
	rsync -avPh \$TMPOUTDIR/ \$OUTDIR/ >> \$LOG 2>&1
	# cp \$TMPOUTDIR/reads1/\$PARSED \$OUTDIR/ >& /dev/null
	# cp \$TMPOUTDIR/\$MIDERRORS \$OUTDIR/ >& /dev/null
	# cp \$TMPOUTDIR/\$REPORT \$OUTDIR/ >& /dev/null
	NTRY=\$((\$NTRY+1))
done
EOF

else

cat >> $SMSJOB <<EOF
PARSED=\$(echo -n \$FQ | perl -pe 's/\.(f(ast)?q).*/\_parsed\.\$1\.bz2/g')
MIDERRORS=\$(echo -n \$FQ | perl -pe 's/\.(f(ast)?q).*/\_miderrors\.\$1\.bz2/g')
REPORT=\$(echo -n \$FQ | perl -pe 's/\.f(ast)?q.*/\_parsed\.report\.txt/g')

NTRY=0
while [[ ! -e \$TMPOUTDIR/\$PARSED || ! -e \$OUTDIR/\$PARSED || \
	     "\$(md5sum \$TMPOUTDIR/\$PARSED | awk '{print \$1}')" != "\$(md5sum \$OUTDIR/\$PARSED | awk '{print \$1}')" ]] \
		 && [[ \$NTRY -lt 10 ]];
do
	sleep \$((30+\$RANDOM%90)) # wait 30-120 secs to try again
	echo "Transferring files to final destination \$OUTDIR" >> \$LOG
	rsync -avPh \$TMPOUTDIR/ \$OUTDIR/ >> \$LOG 2>&1
	# cp \$TMPOUTDIR/\$PARSED \$OUTDIR/ >& /dev/null
	# cp \$TMPOUTDIR/\$MIDERRORS \$OUTDIR/ >& /dev/null
	# cp \$TMPOUTDIR/\$REPORT \$OUTDIR/ >& /dev/null
done
EOF

fi

cat >> $SMSJOB <<EOF
rm -rf \$RUNTMPDIR

echo >> \$LOG
echo "---------------------------------------------------" >> \$LOG
date >> \$LOG
echo >> \$LOG

EOF

chmod +x $SMSJOB
# =============================================================================





# Scripts to summarize results if parallelized
# =============================================================================
if [[ $NPAR -gt 1 ]];
then

# Auxiliary perl script to summarize reports
# .............................................................................
cat > $OUTDIR/merge_reports.pl << 'EOF3'
#!/usr/bin/env perl

use warnings;
use strict;

my @files=`ls $ARGV[0]`;

my $analysed=0;
my $barcoded=0;
my $nobarcoded=0;
my $adapter=0;
my $short=0;

my %sampbcd;
my %sampcts;
foreach my $f (@files){
	chomp($f);
	$analysed+=`sed -n 2p $f | perl -pe 'chomp; s/ sequences.*//g; s/ +//g'`;
	$barcoded+=`sed -n 3p $f | perl -pe 'chomp; s/ sequences.*//g; s/ +//g'`;
	$nobarcoded+=`sed -n 4p $f | perl -pe 'chomp; s/ sequences.*//g; s/ +//g'`;
	$adapter+=`sed -n 5p $f | perl -pe 'chomp; s/ sequences.*//g; s/ +//g'`;
	$short+=`sed -n 6p $f | perl -pe 'chomp; s/ sequences.*//g; s/ +//g'`;

	open (F, "tail -n+11 $f |");
		while (<F>){
			my @aux=split(/,/,$_);
			$sampbcd{$aux[0]}=$aux[1];
			$sampcts{$aux[0]}+=$aux[2];
		}
	close (F);
}
print `head -n1 $files[0]`; # print header

print "$analysed sequences analysed\n";
print "$barcoded sequences contained barcodes\n";
print "$nobarcoded sequences didn't contain barcodes\n";
print "$adapter sequences may have a 3' adapter\n";
print "$short sequences were too short after removing the 3' adapter (<10 + 6 bp)\n";
print "------------------------------------------------------\n\n";

print "barcode_id,barcode,count\n";

foreach my $samp (sort keys %sampcts){
	print "$samp,$sampbcd{$samp},$sampcts{$samp}\n";
}
EOF3
chmod +x $OUTDIR/merge_reports.pl
# .............................................................................


cat > $SMSJOBSUM << EOF2
#!/bin/bash
#$ -hold_jid parse_barcodes.$TIMESTAMP
#$ -l h_rt=$HRS:00:00
#$ -l rmem=${MEM}G
#$ -l mem=${MEM}G
#$ -pe openmp $NTHRSUM
EOF2
if [[ $QUEUE == 'popgenom' || $QUEUE == 'molecol' ]];
then
	echo '#$ -P '$QUEUE >> $SMSJOBSUM
	echo '#$ -q '$QUEUE'.q' >> $SMSJOBSUM
fi

cat >> $SMSJOBSUM << EOF2
#$ -j y
#$ -o $LOGSUM
#$ -V

CLEAN=$CLEAN

EOF2

if [ $PAIREND == 1 ];
then
	# paired end reads
	# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
cat >> $SMSJOBSUM << EOF2
# Reorganize files and directories
# .............................................................................
# parsed split reads
mkdir $OUTDIR/reads1-split-parsed/
mv $OUTDIR/reads1/* $OUTDIR/reads1-split-parsed/
mkdir $OUTDIR/reads2-split-parsed/
mv $OUTDIR/reads2/* $OUTDIR/reads2-split-parsed/

# store logs of split jobs
mkdir $OUTDIR/logs-split
mv $OUTDIR/*[0-9][0-9].log $OUTDIR/logs-split

# store reports of split jobs
mkdir $OUTDIR/reports-split
mv $OUTDIR/*[0-9][0-9]_parsed.report.txt $OUTDIR/reports-split
# .............................................................................

BASEFQS=(\$(ls $OUTDIR/reads1-split-parsed/*parsed* | perl -pe 's/\-[0-9]{2}\_parsed.*//g' | sort -V | uniq))
BASEFQPS=(\$(ls $OUTDIR/reads2-split-parsed/*parsed* | perl -pe 's/\-[0-9]{2}\_parsed.*//g' | sort -V | uniq))

for i in \${!BASEFQS[@]};
do
	# reads 1
	BASE1=\${BASEFQS[\$i]}
	OUTBASE1=$OUTDIR/reads1/\$(basename \$BASE1)	
	
	# without barcodes
	echo -n > \$OUTBASE1'_miderrors.fq' 
	ls \$BASE1*miderrors*.fq.* | sort -V | xargs -I {} sh -c '$COMP $NTHRSUM -dck {} >> '\$OUTBASE1'"_miderrors.fq"' 
	$COMP $NTHRSUM \$OUTBASE1'_miderrors.fq'

	# with barcodes
	echo -n > \$BASE1'_parsed.fq'
	ls \$BASE1*parsed*.fq.* | sort -V | xargs -I {} sh -c '$COMP $NTHRSUM -dck {} >> '\$OUTBASE1'"_parsed.fq"' 
	$COMP $NTHRSUM \$OUTBASE1'_parsed.fq'

	
	# reads2
	BASE2=\${BASEFQPS[\$i]}
	OUTBASE2=$OUTDIR/reads2/\$(basename \$BASE2)

	# without barcodes
	echo -n > \$OUTBASE2'_miderrors.fq'
	ls \$BASE2*miderrors*.fq.* | sort -V | xargs -I {} sh -c '$COMP $NTHRSUM -dck {} >> '\$OUTBASE2'"_miderrors.fq"' 
	$COMP $NTHRSUM -f \$OUTBASE2'_miderrors.fq'

	# with barcodes
	echo -n > \$OUTBASE2'_parsed.fq'
	ls \$BASE2*parsed*.fq.* | sort -V | xargs -I {} sh -c '$COMP $NTHRSUM -dck {} >> '\$OUTBASE2'"_parsed.fq"' 
	$COMP $NTHRSUM -f \$OUTBASE2'_parsed.fq'

	# summarize reports
	$OUTDIR/merge_reports.pl $OUTDIR/reports-split/\$(basename \$BASE1)'-[0-9][0-9]_parsed.report.txt' > \$OUTBASE1"_parsed.report.txt"

done

# clean up
if [[ \$CLEAN == 1 ]];
then
	# find $OUTDIR -type f -regextype posix-extended -regex "$OUTDIR/reads1/\$(basename \$BASE1)-[0-9][0-9]_(miderrors|parsed)_1.fq.(gz|bz2).*" -exec "echo rm -f {}" \;
	# remove auxiliary directories and scripts
	rm -rf $OUTDIR/barcodes-split
	rm -rf $OUTDIR/reads[1-2]-split
	rm -rf $OUTDIR/reads[1-2]-split-parsed
	rm $OUTDIR/merge_reports.pl
fi

EOF2
	# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
else
	# single reads
	# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
cat >> $SMSJOBSUM << EOF2
# Reorganize files and directories
# .............................................................................
# parsed split reads
mkdir $OUTDIR/reads-split-parsed/
mv $OUTDIR/*.fq.* $OUTDIR/reads-split-parsed/

# store logs of split jobs
mkdir $OUTDIR/logs-split
mv $OUTDIR/*[0-9][0-9].log $OUTDIR/logs-split

# store reports of split jobs
mkdir $OUTDIR/reports-split
mv $OUTDIR/*[0-9][0-9]_parsed.report.txt $OUTDIR/reports-split
# .............................................................................

BASEFQS=(\$(ls $OUTDIR/reads-split-parsed/*parsed* | perl -pe 's/\-[0-9]{2}\_parsed.*//g' | sort -V | uniq))

for i in \${!BASEFQS[@]};
do
	BASE=\${BASEFQS[\$i]}
	OUTBASE=$OUTDIR/\$(basename \$BASE)	

	# without barcodes
	echo -n > \$OUTBASE'_miderrors.fq'
	ls \$BASE*miderrors*.fq.* | sort -V | xargs -I {} sh -c '$COMP $NTHRSUM -dck {} >> '\$OUTBASE'"_miderrors.fq"' 
	$COMP $NTHRSUM -f \$OUTBASE'_miderrors.fq'

	# with barcodes
	echo -n > \$OUTBASE'_parsed.fq'
	ls \$BASE*parsed*.fq.* | sort -V | xargs -I {} sh -c '$COMP $NTHRSUM -dck {} >> '\$OUTBASE'"_parsed.fq"' 
	$COMP $NTHRSUM -f \$OUTBASE'_parsed.fq'

	# summarize reports
	$OUTDIR/merge_reports.pl $OUTDIR/reports-split/\$(basename \$BASE)'-[0-9][0-9]_parsed.report.txt' > \$OUTBASE"_parsed.report.txt"

done

# clean up
if [[ \$CLEAN == 1 ]];
then
	# remove auxiliary directories and scripts
	rm -rf $OUTDIR/barcodes-split
	rm -rf $OUTDIR/reads-split
	rm -rf $OUTDIR/reads-split-parsed
	rm $OUTDIR/merge_reports.pl
fi

EOF2
	# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
fi

fi
chmod +x $SMSJOBSUM
# =============================================================================


# Final info and execution commands
# =============================================================================
echo "Command/s to submit the job/s to Iceberg ($QUEUE queue):"
echo
echo "qsub $SMSJOB"
echo
if [[ $NPAR -gt 1 ]];
then
	echo "qsub $SMSJOBSUM"
	echo
fi
# =============================================================================

