#!/bin/bash

# (c) 2020 - Victor Soria-Carrasco
# victor.soria.carrasco@gmail.com
# Last modified: 23/04/2018 23:01:10

# Description:
# Call SNPs using a parallel approach using multiple nodes
# It splits the reference in multiple regions and call SNPs for
# each one of them. 

VERSION='1.0.8-2018.04.23'

CALLER=$(readlink -f $(dirname $0))"/call_SNVs.pl"
SAMTOOLS='/usr/local/extras/Genomics/apps/samtools/1.8/bin/samtools'

# Default values for optional variables
REGIONS=''
CALLMODE='c'
VARONLY='1'
INDELS='0'
MINMQS=20
PVAR='0.05'
EMAIL=''
QUEUE='iceberg'
HRS=24
MEM=4
NJOBS=100
MAXNREGSPERJOB=50

# Show author
# -----------------------------------------------------------------------------
function author {
	echo
	echo "#########################################"
	echo "  $(basename $0)"
	echo "  version $VERSION"
	echo "  (c) Victor Soria-Carrasco"
	echo "  victor.soria.carrasco@gmail.com"
	echo "#########################################"
	echo
}
# -----------------------------------------------------------------------------

# Show usage
# -----------------------------------------------------------------------------
function usage {
	echo
	echo "Usage:"
	echo "  $(basename $0)"
	echo "      -i <input directory>  => Directory with BAM files (one file per sample/individual)"
	echo "      -r <fasta file>       => Reference sequence used for alignment"
	echo "      -o <output directory> => Output folder"
	echo "      -regs <regions file>  => Regions file (optional, use scaffolds/chromosomes grouped for up to $NJOBS jobs otherwise)"
	echo "      -c <c|m>              => bcftools caller; c=consensus, m=multi-allelic (optional, default=$CALLMODE)"
	echo "      -v <0|1>              => call only variants (optional, default=$VARONLY)"
	echo "      -d <0|1>              => call also indels (optional, default=$INDELS)"
	echo "      -s <quality score>    => minimum phred quality score for alignments to be used for calling (optional, default=$MINMQS)"
	echo "      -p <float 0-1>        => prob. of data under the hypothesis that locus is invariant (optional, default=$PVAR)"
	echo "      -t <allocated time>   => Allocated time (in hours) for the analysis (optional: default=$HRS)"
	echo "      -m <allocated memory> => Allocated memory (in gigabytes) for each analysis (optional: default=$MEM)"
	echo "      -q <queue>            => SGE queue: iceberg | popgenom | molecol (optional: default=iceberg)"
	echo "      -e <email>            => Notification email address (default=none)"
	echo "      -h                    => show this help"
	echo ""
	echo "  Example:"
	echo "      $(basename $0) -i input_dir -o output_dir -r reference -q popgenom"
	echo ""
	echo ""
	exit 0
}
# -----------------------------------------------------------------------------

author

# Get options from the command line
# -----------------------------------------------------------------------------
if [ "$#" -ge "6" ]; # min 6 args: 2 for -i <input directory>, 2 for -r <fasta file>, 2 for -o <output directory>
then 
	while [ $# -gt 0 ]; do
		case "$1" in
			-h|-help) usage
					  ;;
			-i)	shift
				INDIR=$(readlink -f $1)
				;;
			-r)	shift
				REFERENCE=$(readlink -f $1)
				;;
			-o)	shift
				OUTDIR=$(readlink -f $1)
				;;
			-regs)	shift
				REGIONS=$(readlink -f $1)
				;;
			-c)	shift
				CALLMODE=$1
				;;
			-v)	shift
				VARONLY=$1
				;;
			-d)	shift
				INDELS=$1
				;;
			-s)	shift
				MINMQS=$1
				;;
			-p)	shift
				PVAR=$1
				;;
			-t)	shift
				HRS=$1
				;;
			-m)	shift
				MEM=$1
				;;
			-q)	shift
				QUEUE=$1
				;;
			-e)	shift
				EMAIL=$1
				;;	
			*)	echo 
				echo "ERROR - Invalid option: $1"
				echo
				usage
				;;
		esac
		shift
	done
else
	usage
fi
# -----------------------------------------------------------------------------

# Check $INDIR, $OUTDIR are $REFERENCE are defined
# -----------------------------------------------------------------------------
if [[ -z $INDIR || -z $OUTDIR || -z $REFERENCE ]]; then
	if [ -z $INDIR ]; then
		echo
		echo "ERROR: You must specify an input directory"
		echo
	fi
	if [ -z $OUTDIR ]; then
		echo
		echo "ERROR: You must specify an output directory"
		echo
	fi
	if [ -z $REFERENCE ]; then
		echo
		echo "ERROR: You must specify a reference file"
		echo
	fi
	usage
fi
# -----------------------------------------------------------------------------

# Check $INDIR and $REFERENCE exist
# -----------------------------------------------------------------------------
if [ ! -d $INDIR ]; then
	echo
	echo "ERROR: I can't find the input directory $INDIR"
	echo
	exit
fi

if [ ! -f $REFERENCE ]; then
	echo
	echo "ERROR: I can't find the reference file $REFERENCE"
	echo
	exit
fi
# -----------------------------------------------------------------------------

# Check $CALLMODE is OK

if [[ $CALLMODE != 'c' && $CALLMODE != 'm' ]];
then
	echo
	echo "ERROR: caller $CALLMODE not recognised (it must be 'c' or 'm')"
	echo
	exit
fi

# Check $VARONLY is OK

if [[ $VARONLY != '0' && $VARONLY != '1' ]];
then
	echo
	echo "ERROR: call only variants option $VARONLY not recognised (it must be '0' or '1')"
	echo
	exit
fi


# Get number of bam files in the input directory
N=$(find $INDIR -maxdepth 1 -name "*.bam" | wc -l | cut -f1 -d" ")

# Check there are bam files in the input directory
# -----------------------------------------------------------------------------
if [ "$N" -eq "0" ]; then
	echo
	echo "ERROR: No BAM files in $INDIR"
	echo
	exit
fi
# -----------------------------------------------------------------------------

# Index reference if it is not indexed
# -----------------------------------------------------------------------------
if [ ! -e $REFERENCE.fai ]; then
	echo
	echo "Building indexes for reference $REFERENCE..."
	echo "(this might take quite a bit...)"
	echo
	$SAMTOOLS faidx $REFERENCE >& $REFERENCE.faidx.log
	echo
	echo "Done. Log file saved in $REFERENCE.faidx.log"
	echo
	echo "----"
	echo
fi
# -----------------------------------------------------------------------------

# Index bam files if they are not indexed
# -----------------------------------------------------------------------------
for bam in $(find $INDIR -maxdepth 1 -name "*.bam");
do
	bai=${bam%.*}.bai
	if [[ ! -e $bam.bai && ! -e $bai ]];
	then
		echo "No index found for $bam, indexing..."
		$SAMTOOLS index $bam
	fi
done
echo
# -----------------------------------------------------------------------------

# Create output directory
# -----------------------------------------------------------------------------
if [ ! -d $OUTDIR ]; then
	mkdir -p $OUTDIR
fi
# -----------------------------------------------------------------------------

# Get all regions from reference if no file is given
# -----------------------------------------------------------------------------
if [[ $REGIONS == ''  ||  ! -f $REGIONS ]];
then
	REFSIZE=$(cat $REFERENCE.fai | awk '{SUM+=$2}END{print SUM}')
	JOBSIZE=$(perl -e 'print int(('$REFSIZE'/'$NJOBS')+1)')
	REGIONS=($(cat $REFERENCE.fai | sort -V | awk '{print $1}'))
	REGSIZES=($(cat $REFERENCE.fai | sort -V | awk '{print $2}'))	
	# echo "Generating regions file (splitting genome in ~$NJOBS regions of ~$JOBSIZE bp)..."
	echo "Generating regions file "
	echo "  splitting genome in chunks:"
	echo "    max size per job:  $JOBSIZE bp"
	echo "    max no regions per job: $MAXNREGSPERJOB"
	echo 

	CUMSIZE=0
	NREGS=0
	echo -n > $OUTDIR/regions
	RC=1
	# for I in $REGIONS;
	for ((i=0; i<${#REGSIZES[*]}; i++));
	do
		# REGSIZE=$(cat $REFERENCE.fai | awk '{if ($1=="'$REG'") print $2}')
		REG=${REGIONS[$i]}
		REGSIZE=${REGSIZES[$i]}
		CUMSIZE=$(($CUMSIZE + $REGSIZE))

		if [[ $CUMSIZE -gt $JOBSIZE || $NREGS -gt $MAXNREGSPERJOB ]];
		then
			echo "$REG" >> $OUTDIR/regions
			# echo "Set of regions size: $CUMSIZE"
			CUMSIZE=0
			NREGS=0
			echo -ne "  number of jobs: $RC\r"
			RC=$(($RC + 1))
		else
			echo -n "$REG," >> $OUTDIR/regions
			NREGS=$((NREGS + 1))
		fi
	done
	
	if [[ $CUMSIZE -gt 0 ]];
	then
		echo -e "  number of jobs: $RC\r"
		echo >> $OUTDIR/regions
	fi
	
	perl -pi -e 's/,$//g' $OUTDIR/regions
	echo
	echo
else
	cp $REGIONS $OUTDIR/regions
fi

REGIONS=$OUTDIR/regions

# number of regions
N=$(cat $REGIONS | wc -l)
# -----------------------------------------------------------------------------

# Make submission file
# -----------------------------------------------------------------------------
SMSJOB="$OUTDIR/var_calling.$(date +%Y%m%d-%H%M%S).smsjob.sh"
LOG="$OUTDIR/var_calling.$(date +%Y%m%d-%H%M%S).smsjob.log"


# Initialize submission script
echo '#!/bin/bash' > $SMSJOB

# SGE OPTIONS
# -----------------------------------------
echo '#$ -l h_rt='$HRS':00:00' >> $SMSJOB
echo '#$ -l rmem='$MEM'G' >> $SMSJOB
echo '#$ -l mem='$MEM'G' >> $SMSJOB
if [ ! -z "$EMAIL" ];
then
	echo '#$ -m bea' >> $SMSJOB
	echo '#$ -M '$EMAIL >> $SMSJOB
fi
if [[ $QUEUE == 'popgenom' || $QUEUE == 'molecol' ]];
then
	echo '#$ -P '$QUEUE >> $SMSJOB
	echo '#$ -q '$QUEUE'.q' >> $SMSJOB
fi
echo '#$ -t 1-'$N >> $SMSJOB
echo '#$ -j y' >> $SMSJOB
echo '#$ -o '$LOG >> $SMSJOB
echo '#$ -V ' >> $SMSJOB
# -----------------------------------------

VARTXT="variants"
if [[ $VARONLY == "0" ]];
then
	VARTXT="$VARTXT and invariants"
fi

cat >> $SMSJOB <<EOF

CALLER="$CALLER"

BAMDIR="$INDIR"
REFERENCE="$REFERENCE"
OUTDIR="$OUTDIR"
REGIONS=(\$(cat $REGIONS))

INDEX=\$((SGE_TASK_ID-1))

REG=\${REGIONS[\$INDEX]}

JOBID="job"\$(printf %04d \$INDEX)
OUTDIR="\$OUTDIR/\$JOBID"
IDLOG="$OUTDIR/\$JOBID.var_calling.log"

hostname > \$IDLOG
uname -a >> \$IDLOG
date >> \$IDLOG
echo "---------------------------------------------------" >> \$IDLOG
echo >> \$IDLOG
echo "Calling $VARTXT for region/s: \$REG..." >> \$IDLOG
echo >> \$IDLOG
echo "CMD: " >> \$IDLOG
echo "    \$CALLER \\\" >> \$IDLOG
echo "    -b \$BAMDIR \\\" >> \$IDLOG
echo "    -r \$REFERENCE \\\" >> \$IDLOG
echo "    -o \$OUTDIR \\\" >> \$IDLOG
echo "    -regs \$REG \\\" >> \$IDLOG
echo "    -n 1 \\\" >> \$IDLOG
echo "    -c $CALLMODE \\\" >> \$IDLOG
echo "    -v $VARONLY \\\" >> \$IDLOG
echo "    -d $INDELS \\\" >> \$IDLOG
echo "    -q $MINMQS \\\" >> \$IDLOG
echo "    -p $PVAR \\\" >> \$IDLOG
echo "    -quiet \\\" >> \$IDLOG
echo "    -clean" >> \$IDLOG
echo  >> \$IDLOG
echo "---------------------------------------------------" >> \$IDLOG
echo >> \$IDLOG

\$CALLER \\
-b \$BAMDIR \\
-r \$REFERENCE \\
-o \$OUTDIR \\
-regs \$REG \\
-n 1 \\
-c $CALLMODE \\
-v $VARONLY \\
-d $INDELS \\
-q $MINMQS \\
-p $PVAR \\
-quiet \\
-clean \\
>> \$IDLOG 2>&1


# Move and rename output
if [[ -e \$OUTDIR/variants.raw.bcf ]];
then
	mv \$OUTDIR/variants.raw.bcf \$OUTDIR/../\$JOBID.variants.raw.bcf
	mv \$OUTDIR/variants.raw.bcf.csi \$OUTDIR/../\$JOBID.variants.raw.bcf.csi
else
	if  [[ -e \$OUTDIR/sites.raw.bcf ]];
	then
		mv \$OUTDIR/sites.raw.bcf \$OUTDIR/../\$JOBID.sites.raw.bcf
		mv \$OUTDIR/sites.raw.bcf.csi \$OUTDIR/../\$JOBID.sites.raw.bcf.csi
	else
		echo "No variants called for this job" >> \$IDLOG
	fi
fi

REPORT=\$(ls \$OUTDIR/*report.txt)
OUTREPORT=\$(basename \$REPORT | perl -pe '\$_='\$JOBID'.".\$_";')
mv \$REPORT \$OUTDIR/../\$OUTREPORT

# Clean
rm -r \$OUTDIR

echo >> \$IDLOG
echo "---------------------------------------------------" >> \$IDLOG
echo >> \$IDLOG
date >> \$IDLOG
echo >> \$IDLOG

EOF

chmod +x $SMSJOB

echo "Command to submit the job to Iceberg ($QUEUE queue):"
echo
echo "qsub $SMSJOB"
echo

