#!/bin/bash
# (c) 2020 - Victor Soria-Carrasco
# victor.soria.carrasco@gmail.com
# Last modified: 23/07/2018 19:22:40

# Description:
# Summarize alleleEst output from multiple runs for multiple populations
# into a BIMBAM format file

VERSION='1.3.1-2017.11.20'

# External programs path and options
ESTPOST='estpost_alleleEst'
# RSCRIPT='Rscript'
RSCRIPT='R --vanilla -f'

# Default values
SNPANNOT=0
NCORES=4

# Temporary directory
TMPDIR="/scratch/tmp_"$USER"_"$(date +%Y%m%d%H%M%s)"/"
if [[ $(hostname) == "node96" ]];
then
	TMPDIR="/local/tmp_"$USER"_"$(date +%Y%m%d%H%M%s)"/"
fi

# Show author
# -----------------------------------------------------------------------------
function author {
	echo
	echo "#########################################"
	echo "  $(basename $0)"
	echo "  version $VERSION"
	echo "  (c) Victor Soria-Carrasco"
	echo "  victor.soria.carrasco@gmail.com"
	echo "#########################################"
	echo
}
# -----------------------------------------------------------------------------

# Show usage
# -----------------------------------------------------------------------------
function usage {
	echo
	echo "Usage:"
	echo "  $(basename $0)"
	echo "      -i <input directory>  => Directory with alleleEst output files (hdf5)"
	echo "      -o <output directory> => Output folder"
	echo "      -n <number> => Number of cores to use"
	echo "      -a => Output SNP annotation files (for gemma)"
	echo "      -h => show this help"
	echo ""
	echo "  Example:"
	echo "      $(basename $0) -i input_dir -o output_dir"
	echo ""
	echo ""
	exit 0
}
# -----------------------------------------------------------------------------

author

# Get options from the command line
# -----------------------------------------------------------------------------
if [ "$#" -ge "4" ]; # min 4 args: 2 for -i <input file>, 2 for -o <output directory>
then 
	while [ $# -gt 0 ]; do
		case "$1" in
			-h|-help) usage
					  ;;
			-i)	shift
				INDIR=$(readlink -f $1)
				;;
			-o)	shift
				OUTDIR=$(readlink -f $1)
				;;
			-n)	shift
				NCORES=$1
				;;
			-a)	SNPANNOT=1
				;;

		*)	echo 
				echo "ERROR - Invalid option: $1"
				echo
				usage
				;;
		esac
		shift
	done
else
	usage
fi
# -----------------------------------------------------------------------------

# Check $INDIR and $OUTDIR are defined
# -----------------------------------------------------------------------------
if [[ -z $INDIR || -z $OUTDIR ]]; then
	if [ -z $INDIR ]; then
		echo
		echo "ERROR: You must specify an input file"
		echo
	fi
	if [ -z $OUTDIR ]; then
		echo
		echo "ERROR: You must specify an output directory"
		echo
	fi
	usage
fi
# -----------------------------------------------------------------------------

# Check $INDIR exist
# -----------------------------------------------------------------------------
if [ ! -e $INDIR ]; then
	echo
	echo "ERROR: I can't find the input file $INDIR"
	echo
	exit
fi
# -----------------------------------------------------------------------------

# Create output directory
# -----------------------------------------------------------------------------
if [ ! -d $OUTDIR ]; then
	mkdir -p $OUTDIR
fi
# -----------------------------------------------------------------------------

# Summarize output using all chains 
# -----------------------------------------------------------------------------
DATASETS=$(find $INDIR -name "*_1.hdf5" | perl -pe 's/\_1\.hdf5//g'| sort -V)
NDATASETS=$(echo $DATASETS | wc -w)
if [ $NDATASETS == 0 ];
then
echo
echo "No hdf5 files found in input directory $INDIR"
	echo
	exit
fi

echo "$NDATASETS datasets found"
echo

for data in $DATASETS;
do
	echo "Summarizing results for dataset \"$(basename $data)\"..."
	echo

	OUTBASE=$OUTDIR/$(basename $data)	
	NRUNS=$(ls $data"_"*.hdf5 | wc -l)
	LOCIFILE=$data".loci"
	SAMPLESFILE=$data".samples"

	# output genotype probabilities
	# ============================================================================
	echo "	Summarizing genotype probabilities..."
	echo
	$ESTPOST \
	-p g \
	-o $OUTBASE.estpost.dsv \
	$data"_"*.hdf5 >& $OUTBASE.estpost.log
	lbzip2 -n $NCORES $OUTBASE.estpost.dsv
	# ============================================================================

	# transpose matrix of genotype probabilities
	# ============================================================================
	echo "	Generating BIMBAM file with genotypes..."
	echo

	# print genotypes for each individual to a temporary file
	# (use of temporary files reduce memory requirements)
	mkdir -p $TMPDIR >& /dev/null
	lbzip2 -n $NCORES -cdk $OUTBASE.estpost.dsv.bz2 | \
	tail -n+3 | \
		awk -F',' 'OFS=" ";
		{
			for (i=1; i<=NF; i++) {
				print $i > "'$TMPDIR'"sprintf("%010d",NR)".tmp"
			}
		}' >& /dev/null

	# New header for debugging
	# echo "locus ref alt "$(cat $SAMPLESFILE | perl -pe 's/\n/ /g') > $OUTBASE.geno

	# Now paste all columns together ignoring old header
	# paste loci, alleles, and genotypes
	paste -d " " $LOCIFILE $TMPDIR/*.tmp | tail -n+2 >> $OUTBASE.geno
	pigz -p $NCORES $OUTBASE.geno
	# ============================================================================

	# SNP annotation file (for gemma)
	# ============================================================================
	if [ $SNPANNOT == 1 ];
	then
		echo "	Generating SNP annotation file..."
		echo
		# if there is a field with lg or chr
		if [ $(grep -chP 'lg|chr' $LOCIFILE) -gt 0 ];
		then
			tail -n+2 $LOCIFILE | \
			perl -pe '/((lg|chr)([0-9]+|NA)\w+\.?[0-9]?\:([0-9]+))/;$_="$1,$4,$3\n";' \
			> $OUTBASE.snpannot
		else
		# look for scaf(fold) field
			tail -n+2 $LOCIFILE | \
			perl -pe '/(\w*scaf[a-z]*([0-9]+\.?[0-9]?)\:([0-9]+))/;$_="$1,$3,$2\n";' \
			> $OUTBASE.snpannot
		fi
	fi
	# ============================================================================

	cp $LOCIFILE $OUTDIR/
	cp $SAMPLESFILE $OUTDIR/

	rm -r $TMPDIR >& /dev/null
done

