#!/bin/bash
# (c) 2020 - Victor Soria-Carrasco
# victor.soria.carrasco@gmail.com
# Last modified: 22/08/2017 14:48:39

# Description:
# Use alleleEst to estimate genotypes

VERSION='1.2.5-2017.08.22'

# External programs path and options
ALLELEEST='alleleEst'
ESTPOST='estpost_alleleEst'
BCFTOOLS='/usr/local/extras/Genomics/apps/bcftools/1.5/bin/bcftools'

# Default values for optional variables
BURNIN=5000
MCMCLEN=10000
SMPFREQ=10
NRUNS=3
EMAIL=''
QUEUE='iceberg'
HRS=48
MEM=2
CLEAN=0
DATE=$(date +%Y%m%d-%H%M%S)

# Show author
# -----------------------------------------------------------------------------
function author {
	echo
	echo "#########################################"
	echo "  $(basename $0)"
	echo "  version $VERSION"
	echo "  (c) Victor Soria-Carrasco"
	echo "  victor.soria.carrasco@gmail.com"
	echo "#########################################"
	echo
}
# -----------------------------------------------------------------------------

# Show usage
# -----------------------------------------------------------------------------
function usage {
	echo
	echo "Usage:"
	echo "  $(basename $0)"
	echo "      -i <input directory>  => Directory with files in vcf/bcf format"
	echo "      -o <output directory> => Output folder"
	echo "      -n <number> => Number of MCMC chains to run for each population (optional: default=$NRUNS)"
	echo "      -l <number> => MCMC chain length (optional: default=$MCMCLEN)"
	echo "      -b <number> => MCMC burnin (optional: default=$BURNIN)"
	echo "      -s <number> => MCMC chain sampling/thinning (optional: default=$SMPFREQ)"
	echo "      -t <allocated time> => Allocated time (in hours) for the analysis (optional: default=$HRS)"
	echo "      -m <allocated memory> => Allocated memory (in gigabytes) for each analysis (optional: default=$MEM)"
	echo "      -q <queue> => SGE queue: iceberg | popgenom | molecol (optional: default=$QUEUE)"
	echo "      -e <email> => Notification email address (default=none)"
	echo "      -c => Clean output (remove auxiliary files)"
	echo "      -h => show this help"
	echo ""
	echo "  Example:"
	echo "      $(basename $0) -i input_dir -o output_dir -q popgenom"
	echo ""
	echo ""
	exit 0
}
# -----------------------------------------------------------------------------

author

# Get options from the command line
# -----------------------------------------------------------------------------
if [ "$#" -ge "4" ]; # min 4 args: 2 for -i <input dir>, 2 for -o <output directory>
then 
	while [ $# -gt 0 ]; do
		case "$1" in
			-h|-help) usage
				;;
			-i)	shift
				INDIR=$(readlink -f $1)
				;;
			-o)	shift
				OUTDIR=$(readlink -f $1)
				;;
			-n)	shift
				NRUNS=$1
				;;
			-l)	shift
				MCMCLEN=$1
				;;
			-b)	shift
				BURNIN=$1
				;;
			-s)	shift
				SMPFREQ=$1
				;;
			-t)	shift
				HRS=$1
				;;
			-m)	shift
				MEM=$1
				;;
			-q)	shift
				QUEUE=$1
				;;
			-e)	shift
				EMAIL=$1
				;;	
			-c)	shift
				CLEAN=1
				;;	
		*)	echo 
				echo "ERROR - Invalid option: $1"
				echo
				usage
				;;
		esac
		shift
	done
else
	usage
fi
# -----------------------------------------------------------------------------

# Check $INFILE, $POPFILE and $OUTDIR are defined
# -----------------------------------------------------------------------------
if [[ -z $INDIR || -z $OUTDIR ]]; then
	if [ -z $INDIR ]; then
		echo
		echo "ERROR: You must specify an input directory"
		echo
	fi
	if [ -z $OUTDIR ]; then
		echo
		echo "ERROR: You must specify an output directory"
		echo
	fi
	usage
fi
# -----------------------------------------------------------------------------

# Check $INFILE and $POPFILE exist
# -----------------------------------------------------------------------------
if [ ! -d $INDIR ]; then
	echo
	echo "ERROR: I can't find the input directory $INDIR"
	echo
	exit
fi
# -----------------------------------------------------------------------------

# Get vcf or bcf files from input directory
# -----------------------------------------------------------------------------
BCFFILES=$(find -L $INDIR -maxdepth 1 -type f -exec basename {} \; | \
			sort -V | \
			grep -P "\.[b|v]cf(\.gz)?$" | \
			perl -pe 's/\n/\ /g' | perl -pe 's/\ $//g')
NBCFFILES=$(echo $BCFFILES | wc -w)

if [ $NBCFFILES == 0 ];
then
	echo
	echo "ERROR: No vcf/bcf files found in input directory"
	echo
	exit
fi
# -----------------------------------------------------------------------------

# Create output directory
# -----------------------------------------------------------------------------
if [ ! -d $OUTDIR ]; then
	mkdir -p $OUTDIR
fi
# -----------------------------------------------------------------------------

# Make list of samples and loci
# -----------------------------------------------------------------------------
echo "Generating samples and loci files..."
echo
for BCF in $BCFFILES;
do
	SAMPLESFILE=$OUTDIR/$(basename $BCF | perl -pe 's/\.[b|v]cf(\.gz)?/\.samples/g')
	$BCFTOOLS view $INDIR/$BCF | \
	grep -m1 'CHROM' | cut -f10- | \
	perl -pe 's/\t/\n/g' \
	> $SAMPLESFILE

	LOCIFILE=$OUTDIR/$(basename $BCF | perl -pe 's/\.[b|v]cf(\.gz)?/\.loci/g')
	echo "locus ref alt" > $LOCIFILE
	$BCFTOOLS view $INDIR/$BCF | \
	awk '$5!~/,/' | grep -v "^#" | \
	awk '{print $1":"$2,$4,$5}' \
	>> $LOCIFILE
done
# -----------------------------------------------------------------------------


# Memory
# -----------------------------------------------------------------------------
VMEM=`expr $MEM + 2`
# -----------------------------------------------------------------------------

# Number of jobs
# -----------------------------------------------------------------------------
NJOBS=`expr $NRUNS \* $NBCFFILES`
# -----------------------------------------------------------------------------

# Set seeds for all runs
# -----------------------------------------------------------------------------
N=0
SEEDS=''
S1=$RANDOM
S2=$S1
while [ $N -lt $NJOBS ];
do
	while [ $S1 -eq $S2 ];
	do
		S2=$RANDOM
	done
	S1=$S2
	SEEDS="$SEEDS $S2"
	let "N = $N + 1"
done
# -----------------------------------------------------------------------------


# Make submission file
# -----------------------------------------------------------------------------
SMSJOB="$OUTDIR/alleleEst.$DATE.smsjob.sh"
LOG="$OUTDIR/alleleEst.$DATE.smsjob.log"


# Initialize submission script
echo '#!/bin/bash' > $SMSJOB

# SGE OPTIONS
# -----------------------------------------
echo '#$ -l h_rt='$HRS':00:00' >> $SMSJOB
echo '#$ -l rmem='$MEM'G' >> $SMSJOB
echo '#$ -l mem='$VMEM'G' >> $SMSJOB
if [ ! -z "$EMAIL" ];
then
	echo '#$ -m bea' >> $SMSJOB
	echo '#$ -M '$EMAIL >> $SMSJOB
fi
if [[ $QUEUE == 'popgenom' || $QUEUE == 'molecol' ]];
then
	echo '#$ -P '$QUEUE >> $SMSJOB
	echo '#$ -q '$QUEUE'.q' >> $SMSJOB
else
	echo '#$ -l scratch=190G' >> $SMSJOB
fi
if [[ $NRUNS -gt "1" ]];
then
	echo '#$ -t 1-'$NJOBS >> $SMSJOB
fi
echo '#$ -j y' >> $SMSJOB
echo '#$ -o '$LOG >> $SMSJOB
echo >> $SMSJOB
# -----------------------------------------

# Change temporary output dir to /scratch or /local
if [[ $QUEUE == 'popgenom' || $QUEUE == 'molecol' ]];
then
	echo 'mkdir "/local/tmp/$USER-'$DATE'" >& /dev/null' >> $SMSJOB
	echo 'TMPOUTDIR="/local/tmp/$USER-'$DATE'"' >> $SMSJOB
else
	echo 'mkdir "/scratch/$USER-'$DATE'" >& /dev/null' >> $SMSJOB
	echo 'TMPOUTDIR="/scratch/$USER-'$DATE'"' >> $SMSJOB;
fi

cat >> $SMSJOB <<EOF
BCFTOOLS="$BCFTOOLS"

echo "TMPOUTDIR: \$TMPOUTDIR"
SEED=($SEEDS)
INDIR="$INDIR"
BCFFILES=($BCFFILES)
INDEX=\$((\$((SGE_TASK_ID-1)) / $NRUNS))

BCF="\$INDIR/\${BCFFILES[\$INDEX]}"
BCFNAME=\$(basename \$BCF | perl -pe 's/\.[b|v]cf(\.gz)?//g')
RUNINDEX=\$((\$((SGE_TASK_ID-1)) % $NRUNS + 1))

# echo BCF: \$BCF - INDEX: \$INDEX - RUNINDEX: \$RUNINDEX

NSAMPLES=\$(\$BCFTOOLS view \$BCF | grep -m1 CHROM | awk '{print NF-9}')
SAMPLES=\$(\$BCFTOOLS view \$BCF | grep -m1 CHROM | cut -f10- | perl -pe 's/\t/\ /g')
SAMPLESA=\$(printf '1 %.0s' \$(seq 1 \$NSAMPLES) | perl -pe 's/\ $//g')
NLOCI=\$(\$BCFTOOLS view \$BCF | grep -chv "^#")
OUTFILE="\$TMPOUTDIR/\$BCFNAME""_\$RUNINDEX.hdf5"
LOCIFILE="\$TMPOUTDIR/\$BCFNAME""_\$RUNINDEX.loci"
IDLOG="$OUTDIR/\$BCFNAME""_\$RUNINDEX.log"

# echo \$OUTFILE - \$OUTPOST - \$IDLOG


hostname > \$IDLOG
uname -a >> \$IDLOG
date >> \$IDLOG
echo "---------------------------------------------------" >> \$IDLOG
echo >> \$IDLOG
echo "Estimating genotypes with alleleEst from \$BCF..." >> \$IDLOG
echo >> \$IDLOG
echo "CMD: " >> \$IDLOG
echo "	$ALLELEEST \\\" >> \$IDLOG
echo "        -i <(echo \"\$NSAMPLES \$NLOCI\"; \\\" >> \$IDLOG
echo "             echo \"\$SAMPLES\"; \\\" >> \$IDLOG
echo "             echo \"\$SAMPLESA\"; \\\" >> \$IDLOG
echo "             $BCFTOOLS view \$BCF | \\\" >> \$IDLOG
echo "             awk '\\\$5!~/,/' | grep -v \"^#\" | cut -f1,2,10- | \\\" >> \$IDLOG
echo "             perl -pe '" >> \$IDLOG
echo "                 \\\$a='';" >> \$IDLOG
echo "                 while (/\:([0-9]+\,[0-9]+\,[0-9]+)\:/g){ " >> \$IDLOG
echo "                   \\\$a.=\"\\\$1\t\";" >> \$IDLOG
echo "                 }" >> \$IDLOG
echo "                 \\\$a=~ s/\,|\t/ /g;" >> \$IDLOG
echo "                 @aux=split(/\s/,\\\$_);" >> \$IDLOG
echo "                 \\\$_=\"\\\$aux[0]\:\\\$aux[1]\t\\\$a\n\"') \\\ " >> \$IDLOG
echo "        -r \${SEED[\$((SGE_TASK_ID-1))]} \\\" >> \$IDLOG
echo "        -n $MCMCLEN -b $BURNIN -t $SMPFREQ \\\">> \$IDLOG
echo "        -o \$OUTFILE" >> \$IDLOG
echo >> \$IDLOG
echo "---------------------------------------------------" >> \$IDLOG
echo >> \$IDLOG

$ALLELEEST \\
-i <(echo "\$NSAMPLES \$NLOCI"; \\
     echo "\$SAMPLES"; \\
	 echo "\$SAMPLESA"; \\
     \$BCFTOOLS view \$BCF | \\
     awk '\$5!~/,/' | grep -v "^#" | cut -f1,2,10- | \\
     perl -pe '
        \$a="";
        while (/\:([0-9]+\,[0-9]+\,[0-9]+)\:/g){
          \$a.="\$1\t";
        }
        \$a=~ s/\,|\t/ /g;
        @aux=split(/\s/,\$_);
        \$_="\$aux[0]\:\$aux[1]  \$a\n"') \\
-r \${SEED[\$((SGE_TASK_ID-1))]} \\
-n $MCMCLEN -b $BURNIN -t $SMPFREQ \\
-o \$OUTFILE  >> \$IDLOG 2>&1

# Some resilience to frequent fastdata problems
while [ ! -f "$OUTDIR/\$(basename \$OUTFILE)" ];
do
	cp \$OUTFILE $OUTDIR
	sleep \$((30+\$RANDOM%90)) # wait 30-120 secs to try again
done

echo >> \$IDLOG
echo "---------------------------------------------------" >> \$IDLOG
echo >> \$IDLOG
date >> \$IDLOG
echo >> \$IDLOG

EOF

chmod +x $SMSJOB

echo "Command to submit the job to Iceberg ($QUEUE queue):"
echo
echo "qsub $SMSJOB"
echo

