#!/bin/bash

# (c) 2020 - Victor Soria-Carrasco
# victor.soria.carrasco@gmail.com
# Last modified: 23/04/2018 23:56:48

# Description:
# Given an input directory with raw fastq files (gzip and bzip2 compression allowed)
# it will filter sequences by quality

VERSION='2.1.1-2018.04.23'
FILTER="/usr/local/extras/Genomics/apps/bbtools/37.99/bbduk.sh"

# Default values for optional variables
EXTRA=''
EMAIL=''
QUEUE='iceberg'
MINAVGQS=20
MINTRIMQS=20
MINLEN=50
MAXN=10000
INQSCODE=auto
OUTQSCODE=auto
STANDARDCONT=
CUSTOMCONT=
NTHR=1
HRS=24
MEM=4

function author {
	echo
	echo "#########################################"
	echo "  $(basename $0)"
	echo "  version $VERSION"
	echo "  (c) Victor Soria-Carrasco"
	echo "  victor.soria.carrasco@gmail.com"
	echo "#########################################"
	echo
}

function usage {
	echo
	echo "Usage:"
	echo "  $(basename $0)"
	echo "      -i <input directory> => Folder with fastq files (.gz and .bz2 accepted)"
	echo "      -p <input directory> => Folder with paired-end fastq files (.gz and .bz2 accepted) (optional)"
	echo "      -o <output directory> => Output folder to save filtered files"
	echo "      -sc <adapters|artifacts|phix> => Standard adapters/contaminants, separated by commas (optional, default=none)"
	echo "      -cc <adapter/contaminants file> => Files with adapters and/or contaminants, separated by commas (optional,default=none)"
	echo "      -mr <minimum average read quality score> => optional, default=$MINAVGQS"
	echo "      -mb <minimum base quality score> (trim down from the end) => optional, default=$MINTRIMQS"
	echo "      -ml <minimum read length after trimming low quality bases> => optional, default=$MINLEN"
	echo "      -mn <maximum number of Ns> => optional, default=$MAXN"
	echo "      -ic <auto | 64 (Illumina 1.5) | 33 (Illumina 1.9)> => Input QS encoding (optional, default=$INQSCODE)"
	echo "      -oc <auto | 64 (Illumina 1.5) | 33 (Illumina 1.9)> => Output QS encoding (optional, default=$OUTQSCODE)"
	echo "      -n <number of processors> => processors per library (optional, default=$NTHR)"
	echo "      -t <allocated time> => Allocated time (in hours) for each analysis (optional: default=24)"
	echo "      -m <allocated memory> => Allocated memory (in gigabytes) for each analysis (optional: default=4)"
	echo "      -q <queue> => SGE queue: iceberg | popgenom | molecol (optional: default=iceberg)"
	echo "      -e <email> => Notification email address (default=none)"
	echo "      -h => show this help"
	echo ""
	echo "  Example:"
	echo "      $(basename $0) -i raw_reads -o filtered_reads -q popgenom"
	echo ""
	echo ""
	exit 0
}

author

if [ "$#" -ge "6" ]; # min 6 args: 2 for -i <input directory>, 2 for -b <barcodes directory>, 2 for -o <output directory>
then 
	while [ $# -gt 0 ]; do
		case "$1" in
			-h|-help) usage
					  ;;
			-i)	shift
				INDIR=$(readlink -f $1)
				;;
			-p)	shift
				INDIRP=$(readlink -f $1)
				;;
			-sc)shift
				STANDARDCONT=$1
				;;
			-cc)shift
				CUSTOMCONT=$1
				;;
			-o)	shift
				OUTDIR=$(readlink -f $1)
				;;
			-mr)shift
				MINAVGQS=$1
				;;
			-mb)shift
				MINTRIMQS=$1
				;;
			-ml)shift
				MINLEN=$1
				;;
			-mn)shift
				MAXN=$1
				;;
			-ic)shift
				INQSCODE=$1
				;;
			-oc)shift
				OUTQSCODE=$1
				;;
			-n)shift
				NTHR=$1
				;;
			-t)	shift
				HRS=$1
				;;
			-m)	shift
				MEM=$1
				;;
			-q)	shift
				QUEUE=$1
				;;
			-e)	shift
				EMAIL=$1
				;;	
			*)	echo 
				echo "ERROR - Invalid option: $1"
				echo
				usage
				;;
		esac
		shift
	done
else
	usage
fi

PAIREND=0
if [[ ! -z $INDIRP ]]; then
	PAIREND=1
fi

N=$(find $INDIR -maxdepth 1 -name "*.fq*" | wc -l | cut -f1 -d" ")

if [ $PAIREND == 1 ];
then
	NP=$(find $INDIRP -maxdepth 1 -name "*.fq*" | wc -l | cut -f1 -d" ")
	if [[ "$N" -ne "$NP" ]]; then
		echo
		echo $INDIRP
		echo "ERROR: Different number of files for reads 1 ($N) and reads2 ($NP)"
		echo
		exit
	fi
fi

# Memory for java
JAVAMEM=$(($MEM*$NTHR))

# Set temporary directory
if [[ $QUEUE == "popgenom" || $QUEUE == "molecol" ]];
then
	TMPDIR=/local/tmp
else
	TMPDIR=/scratch/tmp
fi

# Set script and log filenames
TIMESTAMP=$(date +%Y%m%d-%H%M%S)
SMSJOB="$OUTDIR/filter_QC.$TIMESTAMP.smsjob.sh"
LOG="$OUTDIR/filter_QC.$TIMESTAMP.smsjob.log"

# all adapters/contaminants
CONT=$(echo "$STANDARDCONT,$CUSTOMCONT" | perl -pe 's/,+/,/g;s/^,//g;s/,$//g')

# Initialize submission script
mkdir -p $OUTDIR
echo '#!/bin/bash' > $SMSJOB

# SGE OPTIONS
# -----------------------------------------
cat >> $SMSJOB <<EOF
#$ -l h_rt=$HRS:00:00
#$ -l rmem=${MEM}G
#$ -l mem=${MEM}G
EOF

if [ ! -z "$EMAIL" ];
then
	echo '#$ -m bea' >> $SMSJOB
	echo "#$ -M $EMAIL" >> $SMSJOB
fi
if [[ $QUEUE == 'popgenom' || $QUEUE == 'molecol' ]];
then
	echo "#$ -P $QUEUE" >> $SMSJOB
	echo "#$ -q ${QUEUE}.q" >> $SMSJOB
fi

cat >> $SMSJOB <<EOF
#$ -pe openmp $NTHR
#$ -t 1-$N
#$ -j y
#$ -o $LOG
#$ -V

EOF
# -----------------------------------------


# set up directories and files
# -----------------------------------------
cat >> $SMSJOB <<EOF
OUTDIR=$OUTDIR
INDEX=\$((SGE_TASK_ID-1))

INDIR=$INDIR
FQFILES=(\$(find \$INDIR -name "*.fq*" -or -name "*.fastq*" | sort -V ))
FQ=\${FQFILES[\$INDEX]}

EOF

# reads 2 if paired-end
# .....................................
if [ $PAIREND == 1 ]; then
cat >> $SMSJOB <<EOF
INDIRP=$INDIRP
FQFILESP=(\$(find \$INDIRP -name "*.fq*" -or -name "*.fastq*" | sort -V ))
FQP=\${FQFILESP[\$INDEX]}

EOF

fi
# .....................................


cat >> $SMSJOB <<EOF

FQB=\$(basename \$FQ)
LOG="\$OUTDIR/"\${FQB%%.*}".log"

# temporary directory for this job
# ..........................................
export TMPDIR="$TMPDIR"
if [[ ! -d $TMPDIR ]];
then
	mkdir -p \$TMPDIR
fi
TIMESTAMP=\$(date +%Y%m%d-%H%M%S)
RUNTMPDIR=\$(mktemp --tmpdir -d \$USER-filter_QC_\$TIMESTAMP.XXXXXXXXXX)

TMPINDIR=\$RUNTMPDIR/in
TMPOUTDIR=\$RUNTMPDIR/out

# Create temporary local directories
if [ ! -e \$TMPINDIR ];
then
	mkdir -p \$TMPINDIR
fi

if [ ! -e \$TMPOUTDIR ];
then
	mkdir -p \$TMPOUTDIR
fi

echo "Using temporary directory \$RUNTMPDIR for job \$INDEX - \$FQ"
# ..........................................

EOF

# copy reads
if [ $PAIREND == 1 ]; then

	cat >> $SMSJOB <<EOF
mkdir -p \$TMPINDIR/reads1
mkdir -p \$TMPINDIR/reads2
mkdir -p \$TMPOUTDIR/reads1
mkdir -p \$TMPOUTDIR/reads2
mkdir -p \$TMPOUTDIR/reads-u

cp \$FQ \$TMPINDIR/reads1/
FQb=\$(basename \$FQ)
cp \$FQP \$TMPINDIR/reads2/
FQPb=\$(basename \$FQP)

FQ=\$TMPINDIR/reads1/\$FQb
FQP=\$TMPINDIR/reads2/\$FQPb
FQOUT=\$TMPOUTDIR/reads1/\$FQb
FQPOUT=\$TMPOUTDIR/reads2/\$FQPb
FQSOUT=\$TMPOUTDIR/reads-u/\$FQb

EOF

else

	cat >> $SMSJOB <<EOF
cp \$FQ \$TMPINDIR/
FQb=\$(basename \$FQ)

FQ=\$TMPINDIR/\$FQb
FQOUT=\$TMPOUTDIR/\$FQb

EOF

fi

# log files
cat >> $SMSJOB <<EOF
# log files
CONTLOG1=\$TMPOUTDIR/\${FQb%%.*}.stats.txt
CONTLOG2=\$TMPOUTDIR/\${FQb%%.*}.refstats.txt
LOG=\$TMPOUTDIR/\${FQb%%.*}.log
EOF

# -----------------------------------------

# header and some info about node
# -----------------------------------------
cat >> $SMSJOB <<EOF

hostname > \$LOG
uname -a >> \$LOG
date >> \$LOG
echo "---------------------------------------------------" >> \$LOG
echo >> \$LOG
echo "Using temporary directory \$RUNTMPDIR" >> \$LOG
echo >> \$LOG
echo "Filtering \$FQ file..." > \$LOG
echo >> \$LOG 

EOF
# -----------------------------------------

# log of executed command
# -----------------------------------------
cat >> $SMSJOB <<EOF
echo "CMD: " >> \$LOG
echo "$FILTER \\\">> \$LOG
echo "-Xmx${JAVAMEM}g \\\" >> \$LOG
echo "threads=$NTHR \\\" >> \$LOG
# echo "monitor=60 \\\" >> \$LOG
EOF

# reads if single, reads1 if paired
# ..........................................
cat >> $SMSJOB <<EOF
echo "in=\$FQ \\\" >> \$LOG
echo "out=\$FQOUT \\\" >> \$LOG
# echo "outm=\$FQ \\\" >> \$LOG
EOF
# ..........................................

# reads2 if paired
# ..........................................
if [ $PAIREND == 1 ]; then
cat >> $SMSJOB <<EOF
echo "in2=\$FQP \\\" >> \$LOG
echo "out2=\$FQPOUT \\\" >> \$LOG
echo "outs=\$FQSOUT \\\" >> \$LOG
# echo "outm2=\$FQP \\\" >> \$LOG
EOF
fi
# ..........................................

# options for adapters/contaminants
# ..........................................
if [ ! -z $CONT ]; then
cat >> $SMSJOB <<EOF
echo "ref=$CONT \\\" >> \$LOG
echo "ktrim=r \\\" >> \$LOG
echo "k=25 \\\" >> \$LOG
echo "rcomp=t \\\" >> \$LOG
echo "hammingdistance=1 \\\" >> \$LOG
echo "mink=11 \\\" >> \$LOG
echo "hammingdistance2=0 \\\" >> \$LOG
echo "stats=\$CONTLOG1\\\" >> \$LOG
echo "refstats=\$CONTLOG2 \\\" >> \$LOG
echo "statscolumns=5 \\\" >> \$LOG
EOF
fi
# ..........................................

# options for quality control
# ..........................................
cat >> $SMSJOB <<EOF
echo "minavgquality=$MINAVGQS \\\" >> \$LOG
echo "qtrim=r \\\" >> \$LOG
echo "trimq=$MINTRIMQS \\\" >> \$LOG
echo "minlength=$MINLEN \\\" >> \$LOG
echo "maxns=$MAXN\\\" >> \$LOG
echo "qin=$INQSCODE \\\" >> \$LOG
echo "qout=$OUTQSCODE \\\" >> \$LOG
echo "ziplevel=9 \\\" >> \$LOG
EOF
# ..........................................

# separator
# ..........................................
cat >> $SMSJOB <<EOF
echo >> \$LOG
echo "---------------------------------------------------" >> \$LOG
echo >> \$LOG

EOF
# ..........................................
# -----------------------------------------


# execute command
# -----------------------------------------
cat >> $SMSJOB <<EOF
cd \$TMPINDIR

$FILTER \\
-Xmx${JAVAMEM}g \\
threads=$NTHR \\
EOF

# reads if single, reads1 if paired
# ..........................................
cat >> $SMSJOB <<EOF
in=\$FQ \\
out=\$FQOUT \\
EOF
# ..........................................

# reads2 if paired
# ..........................................
if [ $PAIREND == 1 ]; then
cat >> $SMSJOB <<EOF
in2=\$FQP \\
out2=\$FQPOUT \\
outs=\$FQSOUT \\
EOF
fi
# ..........................................

# options for adapters/contaminants
# ..........................................
if [ ! -z $CONT ]; then
cat >> $SMSJOB <<EOF
ref=$CONT \\
ktrim=r \\
k=25 \\
rcomp=t \\
hammingdistance=1 \\
hammingdistance2=0 \\
mink=11 \\
stats=\$CONTLOG1 \\
refstats=\$CONTLOG2 \\
statscolumns=5 \\
EOF
fi
# ..........................................

# options for quality control
# ..........................................
cat >> $SMSJOB <<EOF
minavgquality=$MINAVGQS \\
qtrim=r \\
trimq=$MINTRIMQS \\
minlength=$MINLEN \\
maxns=$MAXN \\
qin=$INQSCODE \\
qout=$OUTQSCODE \\
ziplevel=9 \\
>> \$LOG 2>&1

EOF
# ..........................................
# -----------------------------------------

# transfer results to final destination
# -----------------------------------------
if [ $PAIREND == 1 ];
then

cat >> $SMSJOB <<EOF

NTRY=0
while [[ ! -e \$TMPOUTDIR/reads1/\$FQb || ! -e \$OUTDIR/reads1/\$FQb || \
         ! -e \$TMPOUTDIR/reads2/\$FQPb || ! -e \$OUTDIR/reads2/\$FQPb || \
         ! -e \$TMPOUTDIR/reads-u/\$FQb || ! -e \$OUTDIR/reads-u/\$FQb || \
	     "\$(md5sum \$TMPOUTDIR/reads1/\$FQb | awk '{print \$1}')" != "\$(md5sum \$OUTDIR/reads1/\$FQb | awk '{print \$1}')" || \
		 "\$(md5sum \$TMPOUTDIR/reads2/\$FQPb | awk '{print \$1}')" != "\$(md5sum \$OUTDIR/reads2/\$FQPb | awk '{print \$1}')" || \
		 "\$(md5sum \$TMPOUTDIR/reads-u/\$FQb | awk '{print \$1}')" != "\$(md5sum \$OUTDIR/reads-u/\$FQb | awk '{print \$1}')" ]] \
		 && [[ \$NTRY -lt 10 ]];
do
	if [[ \$NTRY -gt 1 ]]; then
		sleep \$((30+\$RANDOM%90)) # wait 30-120 secs to try again
	fi
	echo "Transferring files to final destination \$OUTDIR" >> \$LOG
	rsync -avPh \$TMPOUTDIR/ \$OUTDIR/ >> \$LOG 2>&1
	NTRY=\$((\$NTRY+1))
done
EOF

else

cat >> $SMSJOB <<EOF

NTRY=0
while [[ ! -e \$TMPOUTDIR/\$FQb || ! -e \$OUTDIR/\$FQb || \
	     "\$(md5sum \$TMPOUTDIR/\$FQb | awk '{print \$1}')" != "\$(md5sum \$OUTDIR/\$FQb | awk '{print \$1}')" ]] \
		 && [[ \$NTRY -lt 10 ]];
do
	if [[ \$NTRY -gt 1 ]]; then
		sleep \$((30+\$RANDOM%90)) # wait 30-120 secs to try again
	fi
	echo "Transferring files to final destination \$OUTDIR" >> \$LOG
	rsync -avPh \$TMPOUTDIR/ \$OUTDIR/ >> \$LOG 2>&1

	NTRY=\$((\$NTRY+1))
done
EOF
# -----------------------------------------

# remove temporary directory and finish
# -----------------------------------------
cat >> $SMSJOB <<EOF

rm -f \$RUNTMPDIR

echo >> \$LOG
echo "---------------------------------------------------" >> \$LOG
date >> \$LOG
echo >> \$LOG

EOF
# -----------------------------------------

fi
# -----------------------------------------

chmod +x $SMSJOB


echo "Command to submit the job to Iceberg ($QUEUE queue):"
echo
echo "qsub $SMSJOB"
echo
