#!/usr/bin/env perl 

# (c) 2020 - Victor Soria-Carrasco
# victor.soria.carrasco@gmail.com
# Last modified: 17/08/2017 23:41:33
# This script is based on a script originally written by Tom Parchman

# Description:
# This program removes barcode sequences from fastq reads and places them
# in the name. This program also removes adpater sequences from the 3' end.
# Qual scores corresponding to the removed barcode and adapter are removed 
# as well. Sequences not matching any barcodes are written to a separate
# file (miderrors.txt).
# This now includes a function for correcting barcodes that are off by 1.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# Input files:
# Fastq files can be bzip or bzip2 compressed

# Output files are bziped

# Barcodes file must have the following format
#     barcode_id,barcode
#     2013BSCchum029,ggcaagtca
# 
#	notes:
#     - first line is expected to be the header
#     - commas or tabs can be used as separators
#     - the file can have more than 2 columns, but only the first 2 columns
#       will be used, the others will be ignored
#     - barcodes are the ones in the Illumina sequences 
#       (i.e. reverse complementary sequences of the forward primers in the adapters)

use warnings;
use strict;
use File::Basename;
use File::Path qw(make_path);
use File::Spec;
use Getopt::Long;
use Text::LevenshteinXS qw(distance);
use Term::ProgressBar;

# Harcoded constants
# -------------------------------------------------------------------------------------
my $version='1.2.1-2017.08.17';

# Recognised file extensions
my @fqexts=(".fastq",".fastq.gz", ".fastq.bz2", ".fq",".fq.gz",".fq.bz2");

# 3' adapter: cutsite (Mse1) + protector base + adapter (Illumina PCR primer II)
# see Parchman et al. 2012 supplementary information
my $adapter='TTACAGATCGGAAGAG';

# maximum and minimum barcode length, and range in bp
my $maxbclen=10;
my $minbclen=8;
my $bcrange=$maxbclen-$minbclen;

# Restriction site after barcode
my $ressite='CAATTC'; # inherited from Tom scripts, but maybe wrong?
# my $ressite='GAATTC'; # EcoRI
my $reslen=length($ressite);

# Paired-end reads
my $pairend=0;

# Show/hide progress bar
my $quiet=0;
# -------------------------------------------------------------------------------------

&author;

# Check number of arguments
my ($fq, $fqp, $barcodes, $outdir);
my $ncpu=1;
GetOptions( 
    'f|F=s'  => \$fq,
    'p|P=s'  => \$fqp, 
    'b|B=s'  => \$barcodes, 
    'o|O=s'  => \$outdir,
    'n|N=s'  => \$ncpu, 
    'q|Q'    => \$quiet, 
    'h|help' => \&usage 
);

&usage if (!defined($fq) || !defined($barcodes));

# Paired-end reads?
$pairend=1 if (defined($fqp));

# Input fastq file
my ($base, $dir, $ext)=fileparse($fq, @fqexts);
die ("\nUnrecognised file format (check extension): $fq\n\n") if (!grep(/^$ext$/,@fqexts));

# Output directory
$outdir=$dir if (!defined($outdir));
if (! -e $outdir){
	eval {make_path($outdir)} or die ("\nCan't create output directory: $outdir\n\n");
}
if ($pairend){
	eval {make_path("$outdir/reads1")} or die ("\nCan't create output directory: $outdir/reads1\n\n");
	eval {make_path("$outdir/reads2")} or die ("\nCan't create output directory: $outdir/reads2\n\n");
}

# Get absolute paths
$fq=File::Spec->rel2abs($fq);
$fqp=File::Spec->rel2abs($fqp) if ($pairend);
$barcodes=File::Spec->rel2abs($barcodes);
$outdir=File::Spec->rel2abs($outdir);

# Reads with barcodes
# -------------------
my $in="cat $fq |"; # This eases counting seqs below
# Check if fastq file is compressed
if ($ext =~ /\.gz$/){
	$in="pigz -p $ncpu -dck $fq |";
}
elsif ($ext =~ /\.bz2$/){
	$in="lbzip2 -n $ncpu -dck $fq |";
}

# Paired-end reads
# -------------------
my $inp;
if ($pairend){
	$inp="cat $fqp |"; # This eases counting seqs below
	# Check if fastq file is compressed
	if ($ext =~ /\.gz$/){
		$inp="pigz -p $ncpu -dck $fqp |";
	}
	elsif ($ext =~ /\.bz2$/){
		$inp="lbzip2 -n $ncpu -dck $fqp |";
	}
}
# Input barcode file
open (MIDS, $barcodes)
	or die ("\nCan't open barcodes file $barcodes\n\n");
	my %mids;
	my %midsctr;
	<MIDS>; ## get rid of header in dsv file
	while(<MIDS>){
		chomp;
		my @aux = split (/[,|\t]/, $_);
		$aux[1] =~ tr/[a-z]/[A-Z]/; ## catch lower-case barcode input and make it uppercase
		$aux[1] .= $ressite; # Add restriction site
		$mids{$aux[1]} = $aux[0];
		$midsctr{$aux[1]} = 0; # initialise counters to zero
	}
close (MIDS);
my @barcodes = sort keys %mids;

# Get the number of sequences
my $nseqs=(`$in wc -l`/4);
if ($pairend){
	my $nseqsp=(`$inp wc -l`/4);
	die ("\nERROR:\nnumber of reads 1 != number of reads 2 ==> ".
			"$nseqs != $nseqsp\n\n")
		if ($nseqs != $nseqsp);
}
# Output files
my $parsed="$outdir/".basename($fq, @fqexts)."_parsed.fq.bz2"; # parsed_$infile
my $miderrors="$outdir/".basename($fq, @fqexts)."_miderrors.fq.bz2"; # miderrors_$infile
my $parsedp=my $miderrorsp='';
if ($pairend){
	$parsed="$outdir/reads1/".basename($fq, @fqexts)."_parsed_1.fq.bz2"; # parsed_$infile
	$miderrors="$outdir/reads1/".basename($fq, @fqexts)."_miderrors_1.fq.bz2"; # miderrors_$infile
	$parsedp="$outdir/reads2/".basename($fqp, @fqexts)."_parsed_2.fq.bz2"; # parsed_$infile
	$miderrorsp="$outdir/reads2/".basename($fqp, @fqexts)."_miderrors_2.fq.bz2"; # miderrors_$infile
}

# Print initial summary
if ($pairend){
	print "Fastq input file (reads 1): $fq\n";
	print "Fastq input file (reads 2): $fqp\n";
}
else{
	print "Fastq input file: $fq\n";
}
print "Barcodes file: $barcodes\n"; 
print "No of reads: ".format_bignum($nseqs)."\n";
print "----\n\n";

# Progress bar
# ..................................
my $progress=Term::ProgressBar->new ({
    name=>"Parsing barcodes", 
    count=>$nseqs, 
    ETA=>'linear', 
    term_width=>100,
    silent=>$quiet});

$progress->minor(0);
$progress->max_update_rate(1);
my $next_update=0;
# ..................................

open (INFILE, $in) or die ("\nCan't open input file: $fq\n\n");
open (SEQS, "| lbzip2 -n $ncpu > $parsed") or die ("\nCan't write to output file: $parsed\n\n");
open (ERRORS, "| lbzip2 -n $ncpu > $miderrors") or die ("\nCan't write to output file: $miderrors\n\n");
if ($pairend){
	open (INFILEP, $inp) or die ("\nCan't open paired-end reads input file: $fqp\n\n");
	open (SEQSP, "| lbzip2 -n $ncpu > $parsedp") or die ("\nCan't write to output file: $parsedp\n\n");
	open (ERRORSP, "| lbzip2 -n $ncpu > $miderrorsp") or die ("\nCan't write to output file: $miderrorsp\n\n");
}	
	# my %ilid_bcid; # Store relationship of Illumina machine ids and actual samples ids
	# my $ilid = ''; # Illumina ID, necessary to match paired-end reads
	my $bcfound = 0; # boolean to check if a read has a barcode or not
	my $adacnt = 0;# counter of reads with adapter
	my $adalen = 0;# length of sequence after adapter (incl. adapter)
	my $seqcnt = 0;# counter of sequences
	my $goodmidcnt = 0;# counter of sequences with barcodes
	my $badmidcnt = 0;# counter of sequences without barcodes
	my $shortcnt = 0;# counter of too short sequences
	my $bclen;# barcode length
	my %midscnt;# counter of sequences having each barcode
	my $c=0;
	while (<INFILE>){
		$c++;
		
		my $linep=<INFILEP> if ($pairend); # paired-end line
		
		if (/^[ATCGN]+$/){ # sequence line
			$seqcnt++; # Number of reads in original file
			# print "$seqcnt\n" if(!($seqcnt % 100000));
			&update_bar($seqcnt) if (!$quiet && !($seqcnt % 10000));
			# &update_bar($seqcnt) if (!$quiet);
			
			# Initialise variables
			$bcfound=0;
			$adalen=0;

			chomp;
		
			# Check for adapter seq at end
			if (/$adapter/){
				s/($adapter.*)//;
				$adalen=length($1);
				$adacnt++;
			}
			
			# Look for barcodes
			# ------------------------------------------------------
			if (length($_) <= ($minbclen+$reslen)){
				# not enough remaining sequence to even test for a barcode
				$shortcnt++;
			}
			else{
				# Search exact matches for barcodes of 10, 9, or 8 bp
				for my $i (0..$bcrange){
					$bclen=($maxbclen+$reslen)-$i;
					my $bc=substr($_, 0, ($maxbclen-$i));
					my $bcrs=substr($_, 0, $bclen);
					if ($mids{$bcrs} && length($_) >$bclen){
						$bcfound=1;
						$goodmidcnt++;
						$midscnt{$bcrs}++;
						my $seqline=substr($_, $bclen);
						my $out=my $outp="";
							
						if ($pairend){
							$out.="@".$mids{$bcrs}." -- "."$seqcnt/1\n";
							$out.="$seqline\n";
							$out.="+\n";

							$outp.="@".$mids{$bcrs}." -- "."$seqcnt/2\n";	
							$outp.="$linep";
							$outp.="+\n";
						
							print SEQSP $outp;
						}
						else{
							$out.="@".$mids{$bcrs}." -- "."$seqcnt\n";
							$out.="$seqline\n";
							$out.="+\n";
						}
						print SEQS $out;

						# $ilid_bcid{$ilid}="@".$mids{$bcrs}." -- "."$seqcnt\n";
						last;
					}
				}
				
				if (!$bcfound){
					# Search for off-1 matches for barcodes of 10, 9 or 8 bp
					for my $i (0..$bcrange){
						$bclen=($maxbclen+$reslen)-$i;
						my $bc=substr($_, 0, ($maxbclen-$i));
						my $bcrs=substr($_, 0, $bclen);
						if (length($_) > $bclen){
							my ($cbcrs, $dist)=correctmid($bcrs);
							if ($dist == 1){
								$bcfound=1;
								$goodmidcnt++;
								$midscnt{$cbcrs}++;
								my $seqline=substr($_, $bclen);
								
								my $out=my $outp='';
								if ($pairend){
									$out.="@".$mids{$cbcrs}." -- "."$seqcnt/1\n";
									$out.="$seqline\n";
									$out.="+\n";

									$outp.="@".$mids{$cbcrs}." -- "."$seqcnt/2\n";	
									$outp.="$linep";
									$outp.="+\n";
								
									print SEQSP $outp;
								}
								else{
									$out.="@".$mids{$cbcrs}." -- "."$seqcnt\n";
									$out.="$seqline\n";
									$out.="+\n";
								}
								print SEQS $out;

								last;
							}
						}
					}
				}
				
				if (!$bcfound){
					my $out=my $outp='';
					if ($pairend){
						$out.='@MIDERROR -- '."$badmidcnt/1\n";
						$out.="$_\n";
						$out.="+\n";
					
						$outp.='@MIDERROR -- '."$badmidcnt/2\n";
						$outp.="$linep";
						$outp.="+\n";

						print ERRORSP $outp;
					}
					else{
						$out.='@MIDERROR -- '."$badmidcnt\n";
						$out.="$_\n";
						$out.="+\n";
					}
					print ERRORS $out;
					
					$badmidcnt++;
				}
			}
		} 
		elsif ($c%4==0){ # quality scores
			chomp;
			if ($bcfound){
				# my $qline=substr($_, ($bclen+$reslen), (length($_)-($bclen+$reslen+$adalen)));
				my $qline=substr($_, $bclen, (length($_)-$bclen-$adalen));
				print SEQS "$qline\n";
				print SEQSP "$linep" if ($pairend);
			}
			else{
				print ERRORS "$_\n";
				print ERRORSP "$linep" if ($pairend);
			}
		}
	}
close (INFILE);
close (SEQS);
close (ERRORS);
if ($pairend){
	close (INFILEP);
	close (SEQSP);
	close (ERRORSP); 
}

$progress->update($nseqs) if ($nseqs>$next_update && !$quiet);

my $reportfile="$outdir/".basename($fq, @fqexts)."_parsed.report.txt"; # parsed_$infile
open (REPORT, "> $reportfile") or die "\nCan't write to report file $reportfile\n\n";
	if ($pairend){
		print REPORT "This is a paired-end dataset\n";
	} else {
		print REPORT "This is a single-read dataset\n";
	}

	print REPORT format_bignum($seqcnt)." sequences analysed\n";
	print REPORT format_bignum($goodmidcnt)." sequences contained barcodes\n";
	print REPORT format_bignum($badmidcnt)." sequences didn't contain barcodes\n";
	print REPORT format_bignum($adacnt)." sequences may have a 3' adapter\n";
	print REPORT format_bignum($shortcnt)." sequences were too short after removing the 3' adapter (<$maxbclen + $reslen bp)\n";
	print REPORT "------------------------------------------------------\n\n";
	print REPORT "barcode_id,barcode,count\n";
	foreach my $bc (sort keys %midscnt){
		print REPORT "$mids{$bc},$bc,$midscnt{$bc}\n";
	}
close (REPORT);

print "\n\n";
# print final summary
print "----\n";
print "Output directory: $outdir\n";
if ($pairend){
	print "Output file (barcode found, reads 1): $parsed\n";
	print "Output file (barcode not found, reads 1): $miderrors\n";
	print "Output file (barcode found, reads 2): $parsedp\n";
	print "Output file (barcode not found, reads 2): $miderrorsp\n";
}
else{
	print "Output file (barcode found reads): $parsed\n";
	print "Output file (barcode not found reads): $miderrors\n";
}
print "Report file: $reportfile\n";
print "\n";


# ==============================================================================
# ==============================================================================
# ============================== SUBROUTINES ===================================
# ==============================================================================
# ==============================================================================

# Show copyright
# ==============================================================================
sub author{
    print "\n";
    print "#########################################\n";
    print "  ".basename($0)."\n";
	print "  version $version\n";
    print "  (c) Victor Soria-Carrasco             \n";
    print "  victor.soria.carrasco\@gmail.com      \n";
    print "#########################################\n";
	print "\n";
}
# ==============================================================================

# Show usage
# ==============================================================================
sub usage{
    print "\n";
	print "  Usage:\n";
    print "    ".basename($0)."\n";
	print "      -f <fastq file>\n";
	print "      -p <fastq file (paired-end reads) (optional)>\n";
	print "      -b <barcodes file (dsv)>\n";
	print "      -o <output directory> (optional)\n";
	# print "      -n <number of cores> (optional, default=1)\n";
	print "      -q <switch on quiet mode> (optional)\n";
    print "\n";
    exit;
}
# ==============================================================================

# Correct MID
# ==============================================================================
sub correctmid{
    my $min = 100;
    my $mindex = -1;
    for my $i (0..$#barcodes){
		#my $bc=$barcodes[$i].$ressite;
		my $dist = distance($_[0], $barcodes[$i]);
		if($min > $dist){
			$min = $dist;
			$mindex = $i;
		}
    }
    return($barcodes[$mindex], $min);
}

# Format big numbers separating every 3 digits with a space
# ==============================================================================
sub format_bignum{
	my $num=$_[0];
	1 while $num =~ s/^(-?\d+)(\d{3})/$1 $2/;
	return($num);
}

# Update progress bar
# ==============================================================================
sub update_bar{
    my $c=shift(@_);
    $next_update=$progress->update($c) if ($c>=$next_update);
}
# ==============================================================================


