#!/bin/bash

# (c) 2020 - Victor Soria-Carrasco
# victor.soria.carrasco@gmail.com
# Last modified: 23/08/2017 16:01:45

# Description:
# Given an input directory with fastq files (gzip and bzip2 compression allowed)
# it will demultiplex sequences according to their names

VERSION='2.0-2017.08.23'
SPLITER="$(dirname $0)/split_reads_by_id.pl"

# Default values for optional variables
EXTRA=''
EMAIL=''
QUEUE='iceberg'
HRS=24
MEM=4
NTHR=1
NTHRSUM=4

CLEAN=0

function author {
	echo
	echo "#########################################"
	echo "  $(basename $0)"
	echo "  version $VERSION"
	echo "  (c) Victor Soria-Carrasco"
	echo "  victor.soria.carrasco@gmail.com"
	echo "#########################################"
	echo
}

function usage {
	echo
	echo "Usage:"
	echo "  $(basename $0)"
	echo "      -i <input directory> => Folder with fastq files (.gz and .bz2 accepted)"
	echo "      -p <input directory> => Folder with paired-end fastq files (.gz and .bz2 accepted) (optional)"
	echo "      -u <input directory> => Folder with unpaired fastq files (e.g. after filtering paired-end) (.gz and .bz2 accepted) (optional)"
	echo "      -o <output directory> => Output folder to save individual files"
	echo "      -n <number of threads> => Number of threads used for compression (optional: default=1)"
	echo "      -z <number of threads> => Number of threads used for compression during summary (optional: default=4)"
	echo "      -t <allocated time> => Allocated time (in hours) for each analysis (optional: default=24)"
	echo "      -m <allocated memory> => Allocated memory (in gigabytes) for each analysis (optional: default=4)"
	echo "      -q <queue> => SGE queue: iceberg | popgenom | molecol (optional: default=iceberg)"
	echo "      -e <email> => Notification email address (default=none)"
	echo "      -c => Clean intermediate files (default=no)"
	echo "      -h => show this help"
	echo ""
	echo "  Example:"
	echo "      $(basename $0) -i parsed_reads -o ids_reads -q popgenom"
	echo ""
	echo ""
	exit 0
}

author

if [ "$#" -ge "4" ]; # min 6 args: 2 for -i <input directory>, 2 for -o <output directory>
then 
	while [ $# -gt 0 ]; do
		case "$1" in
			-h|-help) usage
					  ;;
			-i)	shift
				INDIR=$(readlink -f $1)
				;;
			-p)	shift
				INDIRP=$(readlink -f $1)
				;;
			-u)	shift
				INDIRU=$(readlink -f $1)
				;;
			-o)	shift
				OUTDIR=$(readlink -f $1)
				;;
			-n)	shift
				NTHR=$1
				;;
			-z)	shift
				NTHRSUM=$1
				;;
			-t)	shift
				HRS=$1
				;;
			-m)	shift
				MEM=$1
				;;
			-q)	shift
				QUEUE=$1
				;;
			-e)	shift
				EMAIL=$1
				;;	
			-c)	CLEAN=1
				;;
			*)	echo 
				echo "ERROR - Invalid option: $1"
				echo
				usage
				;;
		esac
		shift
	done
else
	usage
fi

PAIREND=0
UNPAIRED=0
if [[ ! -z $INDIRP ]]; then
	PAIREND=1
	if [[ ! -z $INDIRU ]]; then
		UNPAIRED=1
	fi
fi

N=$(find $INDIR -maxdepth 1 -name "*parsed.fq*" | wc -l | cut -f1 -d" ")

if [ $PAIREND == 1 ];
then
	NP=$(find $INDIRP -maxdepth 1 -name "*parsed.fq*" | wc -l | cut -f1 -d" ")
	if [[ "$N" -ne "$NP" ]]; then
		echo
		echo $INDIRP
		echo "ERROR: Different number of files for reads 1 ($N) and reads2 ($NP)"
		echo
		exit
	fi

	if [ $UNPAIRED == 1 ];
	then
		NU=$(find $INDIRU -maxdepth 1 -name "*parsed.fq*" | wc -l | cut -f1 -d" ")
	else
		NU=0
	fi

	# sum pair and paired end together (and unpaired if given)
	N=$(( $N + $NP + $NU ))
fi

# Set temporary directory
if [[ $QUEUE == "popgenom" || $QUEUE == "molecol" ]];
then
	TMPDIR=/local/tmp
else
	TMPDIR=/scratch/tmp
fi

# Set script and log filenames
TIMESTAMP=$(date +%Y%m%d-%H%M%S)
SMSJOB="$OUTDIR/split_reads_by_id.$TIMESTAMP.smsjob.sh"
LOG="$OUTDIR/split_reads_by_id.$TIMESTAMP.smsjob.log"
SMSJOBSUM="$OUTDIR/split_reads_by_id.$TIMESTAMP.smsjobsum.sh"
LOGSUM="$OUTDIR/split_reads_by_id.$TIMESTAMP.smsjobsum.log"

# Initialize submission script
mkdir -p $OUTDIR

# submission script
# =============================================================================
# SGE OPTIONS
# -----------------------------------------
cat << EOF > $SMSJOB
#!/bin/bash
#$ -N split_reads_by_id.$TIMESTAMP
#$ -l h_rt=$HRS:00:00
#$ -l rmem=${MEM}G
#$ -l mem=${MEM}G
EOF
if [ ! -z "$EMAIL" ];
then
	echo '#$ -m bea' >> $SMSJOB
	echo '#$ -M '$EMAIL >> $SMSJOB
fi
if [[ $QUEUE == 'popgenom' || $QUEUE == 'molecol' ]];
then
	echo '#$ -P '$QUEUE >> $SMSJOB
	echo '#$ -q '$QUEUE'.q' >> $SMSJOB
fi

cat << EOF >> $SMSJOB
#$ -pe openmp $NTHR
#$ -t 1-$N
#$ -j y
#$ -o $LOG
#$ -V

EOF
# -----------------------------------------

cat >> $SMSJOB <<EOF
INDIR=$INDIR
OUTDIR=$OUTDIR
FQFILES=(\$INDIR/*parsed.fq*)

EOF

if [ $PAIREND == 1 ]; then

cat >> $SMSJOB <<EOF
# store index - says if they are reads1 or reads2
READTYPE=(\$(printf '%0.s1 ' \$(seq 1 \${#FQFILES[@]})))

INDIRP=$INDIRP
FQFILESP=(\$INDIRP/*parsed.fq*)
FQFILES+=( "\${FQFILESP[@]}" )
READTYPE+=( \$(printf '%0.s2 ' \$(seq 1 \${#FQFILESP[@]})) )

EOF
	if [ $UNPAIRED == 1 ]; then
	
cat >> $SMSJOB <<EOF
INDIRU=$INDIRU
FQFILESU=(\$INDIRU/*parsed.fq*)
FQFILES+=( "\${FQFILESU[@]}" )
READTYPE+=( \$(printf '%0.s-u ' \$(seq 1 \${#FQFILESU[@]})) )

EOF

	fi
fi


cat >> $SMSJOB <<EOF

INDEX=\$((SGE_TASK_ID-1))
FQ=\${FQFILES[\$INDEX]}

# temporary directory for this job
export TMPDIR="$TMPDIR"
if [[ ! -d \$TMPDIR ]];
then
	mkdir -p \$TMPDIR
fi
TIMESTAMP=\$(date +%Y%m%d-%H%M%S)
RUNTMPDIR=\$(mktemp --tmpdir -d \$USER-parse_barcodes_\$TIMESTAMP.XXXXXXXXXX)
TMPID=\$(basename \$RUNTMPDIR)

TMPINDIR=\$RUNTMPDIR/in
TMPOUTDIR=\$RUNTMPDIR/out

if [[ ! -z \$READTYPE ]];
then
	TMPOUTDIR=\$TMPOUTDIR/reads\${READTYPE[\$INDEX]}
fi

# debug
# echo \${READTYPE[@]}
# echo \${FQFILES[@]}
# echo \$INDEX \$FQ \$TMPINDIR \$TMPOUTDIR

# Create temporary local directories
if [ ! -e \$TMPINDIR ];
then
	mkdir -p \$TMPINDIR
fi

if [ ! -e \$TMPOUTDIR ];
then
	mkdir -p \$TMPOUTDIR
fi

# copy to local temporary directory
cp \$FQ \$TMPINDIR/
FQ=\$(basename \$FQ)

LOG="\$OUTDIR/\${FQ%%.*}.log"

echo "Splitting \$FQ file by sample/individual..." > \$LOG
echo >> \$LOG 
echo "CMD: " >> \$LOG
echo "$SPLITER \\\" >> \$LOG
echo "-i \$FQ \\\" >> \$LOG
echo "-o \$OUTDIR \\\" >> \$LOG
echo "-n $NTHR" \\\ >> \$LOG
echo "-f \$((\$INDEX+1))" >> \$LOG
echo >> \$LOG
echo "---------------------------------------------------" >> \$LOG
echo >> \$LOG

cd \$TMPINDIR

$SPLITER \\
-i \$FQ \\
-o \$TMPOUTDIR \\
-n $NTHR \\
-f \$((\$INDEX+1)) \\
>> \$LOG 2>&1

cd \$TMPOUTDIR
md5sum * > md5sumcheck

# Copy results back to output directory
# and check all files are good
OUTDIR=\$OUTDIR/tmp/reads\${READTYPE[\$INDEX]}/\$TMPID

mkdir -p \$OUTDIR
cp -r \$TMPOUTDIR/* \$OUTDIR/ >& /dev/null
cd \$OUTDIR

NTRY=0
while [[ ! -e \$TMPOUTDIR || \$(md5sum -c md5sumcheck | grep -c "OK") != \$(cat md5sumcheck | wc -l) ]] \
		 && [[ \$NTRY -lt 10 ]];
do
	sleep \$((30+\$RANDOM%90)) # wait 30-120 secs to try again
	rsync -avPh \$TMPOUTDIR/* \$OUTDIR/ >& /dev/null
	NTRY=\$((\$NTRY+1))
done

rm -rf \$RUNTMPDIR 

EOF

chmod +x $SMSJOB
# =============================================================================





# Summarize script
# =============================================================================
# SGE OPTIONS
# -----------------------------------------
cat > $SMSJOBSUM << EOF2
#!/bin/bash
#$ -hold_jid split_reads_by_id.$TIMESTAMP
#$ -l h_rt=$HRS:00:00
#$ -l rmem=${MEM}G
#$ -l mem=${MEM}G
EOF2

if [ ! -z "$EMAIL" ];
then
	echo '#$ -m bea' >> $SMSJOBSUM
	echo '#$ -M '$EMAIL >> $SMSJOBSUM
fi

if [[ $QUEUE == 'popgenom' || $QUEUE == 'molecol' ]];
then
	echo '#$ -P '$QUEUE >> $SMSJOBSUM
	echo '#$ -q '$QUEUE'.q' >> $SMSJOBSUM
fi

cat >> $SMSJOBSUM << EOF2
#$ -pe openmp $NTHRSUM
#$ -j y
#$ -o $LOGSUM
#$ -V
EOF2
# -----------------------------------------

cat >> $SMSJOBSUM << EOF2

INDIR=$OUTDIR/tmp
OUTDIR=$OUTDIR
NTHRSUM=$NTHRSUM
CLEAN=$CLEAN

EOF2


cat >> $SMSJOBSUM << 'EOF2'

# function to merge files of same sample/individual
# --------------------------------------------------------------------------
merge_id(){
	ID=$1
	INDIR=$2
	OUTDIR=$3

	FLS=($(find $INDIR -name "$ID.*" | grep -P "$ID\.f(ast)?q" | sort -V))
	EXT=${FLS[0]##*.}
	
	echo "Sample/individual id: $ID ($EXT)"
	
	if [[ $EXT == "gz" ]];
	then
		gzip -cd ${FLS[@]} | gzip > $OUTDIR/$ID.fq.gz
	elif [[ $EXT == "bz2" ]];
	then
		bzip2 -cd ${FLS[@]} | bzip2 > $OUTDIR/$ID.fq.bz2
	else
		cat ${FLS[@]} > $OUTDIR/$ID.fq
	fi
}

export -f merge_id
# --------------------------------------------------------------------------

EOF2

if [[ $PAIREND == 1 ]];
then
# .............................................................................
cat >> $SMSJOBSUM << 'EOF2'
mkdir -p $OUTDIR/reads1
mkdir -p $OUTDIR/reads2

IDS=($(find $INDIR/reads1  -type f -regextype posix-extended -regex ".*f(ast)?q(.gz|.bz2)" | perl -pe 's/.*\///g; s/\.f(ast)?q.*//g' | sort -V | uniq))
IDSP=($(find $INDIR/reads2  -type f -regextype posix-extended -regex ".*f(ast)?q(.gz|.bz2)" | perl -pe 's/.*\///g; s/\.f(ast)?q.*//g' | sort -V | uniq))

if [[ -e $INDIR/reads-u ]];
then
	mkdir -p $OUTDIR/reads-u
	IDSU=($(find $INDIR/reads-u  -type f -regextype posix-extended -regex ".*f(ast)?q(.gz|.bz2)" | perl -pe 's/.*\///g; s/\.f(ast)?q.*//g' | sort -V | uniq))
fi

echo "Processing ${#IDS[@]} samples/individuals..."
echo

parallel -j $NTHRSUM merge_id ::: ${IDS[@]} ::: $INDIR/reads1 ::: $OUTDIR/reads1
parallel -j $NTHRSUM merge_id ::: ${IDSP[@]} ::: $INDIR/reads2 ::: $OUTDIR/reads2

if [[ ${#IDSU[@]} -gt 0 ]];
then
	parallel -j $NTHRSUM merge_id ::: ${IDSU[@]} ::: $INDIR/reads-u ::: $OUTDIR/reads-u
fi

# count reads (paired)
# .............................................................................
find $OUTDIR/reads1 -type f -regextype posix-extended -regex ".*f(ast)?q(.gz|.bz2)" | \
parallel -j $NTHRSUM '
	IN={}
	EXT=${IN##*.}
	if [[ $EXT == "gz" ]]; then COMP="gzip -cd";
	elif [[ $EXT == "bz2" ]]; then COMP="bzip2 -cd";
	else COMP="cat";
	fi
	ID=$(basename $IN)
	echo -ne "${ID%%.*}\t"
	$COMP $IN | awk "END{\$1=NR/4;print \$1;}"' | \
sort -V > $OUTDIR/tmp-rds1.report.txt
# .............................................................................

# count bp (paired)
# .............................................................................
find $OUTDIR/reads1 -type f -regextype posix-extended -regex ".*f(ast)?q(.gz|.bz2)" | \
parallel -j $NTHRSUM '
	IN={}
	EXT=${IN##*.}
	if [[ $EXT == "gz" ]]; then COMP="gzip -cd";
	elif [[ $EXT == "bz2" ]]; then COMP="bzip2 -cd";
	else COMP="cat";
	fi
	ID=$(basename $IN)
	echo -ne "${ID%%.*}\t"
	$COMP $IN | sed -n '2~4p' | tr -d '\n' | wc -m' | \
sort -V > $OUTDIR/tmp-bps1.report.txt

find $OUTDIR/reads2 -type f -regextype posix-extended -regex ".*f(ast)?q(.gz|.bz2)" | \
parallel -j $NTHRSUM '
	IN={}
	EXT=${IN##*.}
	if [[ $EXT == "gz" ]]; then COMP="gzip -cd";
	elif [[ $EXT == "bz2" ]]; then COMP="bzip2 -cd";
	else COMP="cat";
	fi
	ID=$(basename $IN)
	echo -ne "${ID%%.*}\t"
	$COMP $IN | sed -n '2~4p' | tr -d '\n' | wc -m' | \
sort -V > $OUTDIR/tmp-bps2.report.txt
# .............................................................................

if [[ ${#IDSU[@]} -gt 0 ]];
then

# count reads (unpaired)
# .............................................................................
find $OUTDIR/reads-u -type f -regextype posix-extended -regex ".*f(ast)?q(.gz|.bz2)" | \
parallel -j $NTHRSUM '
	IN={}
	EXT=${IN##*.}
	if [[ $EXT == "gz" ]]; then COMP="gzip -cd";
	elif [[ $EXT == "bz2" ]]; then COMP="bzip2 -cd";
	else COMP="cat";
	fi
	ID=$(basename $IN)
	echo -ne "${ID%%.*}\t"
	$COMP $IN | awk "END{\$1=NR/4;print \$1;}"' | \
sort -V > $OUTDIR/tmp-rds-u.report.txt
# .............................................................................

# count bps (unpaired)
# .............................................................................
find $OUTDIR/reads-u -type f -regextype posix-extended -regex ".*f(ast)?q(.gz|.bz2)" | \
parallel -j $NTHRSUM '
	IN={}
	EXT=${IN##*.}
	if [[ $EXT == "gz" ]]; then COMP="gzip -cd";
	elif [[ $EXT == "bz2" ]]; then COMP="bzip2 -cd";
	else COMP="cat";
	fi
	ID=$(basename $IN)
	echo -ne "${ID%%.*}\t"
	$COMP $IN | sed -n '2~4p' | tr -d '\n' | wc -m' | \
sort -V > $OUTDIR/tmp-bps-u.report.txt
# .............................................................................

fi

# join all info into a single file
if [[ ${#IDSU[@]} -gt 0 ]];
then
	echo -e "id\t#reads1\t#reads2\t#reads-u\t#reads_total\t#bps1\t#bps2\t#bps-u\t#bps_total" > $OUTDIR/split_reads_by_id.report.dsv # header

	join -j1 -t $'\t' <(join -j1 -t $'\t' <(sort -k1V $OUTDIR/tmp-rds1.report.txt) <(sort -k1V  $OUTDIR/tmp-rds1.report.txt)) <(sort -k1V $OUTDIR/tmp-rds-u.report.txt) > $OUTDIR/tmp-rds-all.report.txt
	cat $OUTDIR/tmp-rds-all.report.txt | awk '{OFS="\t"; OUT=$2+$3+$4; print $0,OUT}' >> $OUTDIR/tmp-reads.report.txt

	join -j1 -t $'\t' <(join -j1 -t $'\t' <(sort -k1V $OUTDIR/tmp-bps1.report.txt) <(sort -k1V  $OUTDIR/tmp-bps2.report.txt)) <(sort -k1V $OUTDIR/tmp-bps-u.report.txt) > $OUTDIR/tmp-bps-all.report.txt
	cat $OUTDIR/tmp-bps-all.report.txt | awk '{OFS="\t"; OUT=$2+$3+$4; print $0,OUT}' >> $OUTDIR/tmp-bps.report.txt

	join -j1 -t $'\t' <(sort -k1V $OUTDIR/tmp-reads.report.txt) <(sort -k1V $OUTDIR/tmp-bps.report.txt) >> $OUTDIR/split_reads_by_id.report.dsv
else
	echo -e "id\t#reads1\t#reads2\t#reads_total\t#bps1\t#bps2\t#bps_total" > $OUTDIR/split_reads_by_id.report.dsv # header

	join -j1 -t $'\t' <(sort -k1V $OUTDIR/tmp-rds1.report.txt) <(sort -k1V  $OUTDIR/tmp-rds1.report.txt) > $OUTDIR/tmp-rds-all.report.txt
	cat $OUTDIR/tmp-rds-all.report.txt | awk '{OFS="\t"; OUT=$2+$3; print $0,OUT}' >> $OUTDIR/tmp-reads.report.txt

	join -j1 -t $'\t' <(sort -k1V $OUTDIR/tmp-bps1.report.txt) <(sort -k1V  $OUTDIR/tmp-bps2.report.txt) > $OUTDIR/tmp-bps-all.report.txt
	cat $OUTDIR/tmp-bps-all.report.txt | awk '{OFS="\t"; OUT=$3+$4; print $0,OUT}' >> $OUTDIR/tmp-bps.report.txt

	join -j1 -t $'\t' <(sort -k1V $OUTDIR/tmp-reads.report.txt) <(sort -k1V $OUTDIR/tmp-bps.report.txt) >> $OUTDIR/split_reads_by_id.report.dsv
fi

rm $OUTDIR/tmp*.report.txt

EOF2
# .............................................................................
else
# single reads
# .............................................................................
cat >> $SMSJOBSUM << 'EOF2'
mkdir -p $OUTDIR/reads

IDS=($(find $INDIR  -type f -regextype posix-extended -regex ".*f(ast)?q(.gz|.bz2)" | perl -pe 's/.*\///g; s/\.f(ast)?q.*//g' | sort -V | uniq))
echo "Processing ${#IDS[@]} samples/individuals..."
echo

parallel -j $NTHRSUM merge_id ::: ${IDS[@]} ::: $INDIR ::: $OUTDIR/reads

# count reads
# .............................................................................
find $OUTDIR/reads  -type f -regextype posix-extended -regex ".*f(ast)?q(.gz|.bz2)" | \
parallel -j $NTHRSUM '
	IN={}
	EXT=${IN##*.}
	if [[ $EXT == "gz" ]]; then COMP="gzip -cd";
	elif [[ $EXT == "bz2" ]]; then COMP="bzip2 -cd";
	else COMP="cat";
	fi
	ID=$(basename $IN)
	echo -ne "${ID%%.*}\t"
	$COMP $IN | awk "END{\$1=NR/4;print \$1;}"' | \
sort -V > $OUTDIR/tmp-reads.report.txt
# .............................................................................

# count bps
# .............................................................................
find $OUTDIR/reads -type f -regextype posix-extended -regex ".*f(ast)?q(.gz|.bz2)" | \
parallel -j $NTHRSUM '
	IN={}
	EXT=${IN##*.}
	if [[ $EXT == "gz" ]]; then COMP="gzip -cd";
	elif [[ $EXT == "bz2" ]]; then COMP="bzip2 -cd";
	else COMP="cat";
	fi
	ID=$(basename $IN)
	echo -ne "${ID%%.*}\t"
	$COMP $IN | sed -n '2~4p' | tr -d '\n' | wc -m' | \
sort -V > $OUTDIR/tmp-bps.report.txt
# .............................................................................

echo -e "id\t#reads\t#bps" > $OUTDIR/split_reads_by_id.report.dsv # header

join -j1 -t $'\t' <(sort -k1V $OUTDIR/tmp-reads.report.txt) <(sort -k1V $OUTDIR/tmp-bps.report.txt) >> $OUTDIR/split_reads_by_id.report.dsv

rm $OUTDIR/tmp*.report.txt

EOF2
# .............................................................................
fi

cat >> $SMSJOBSUM << 'EOF2'
if [[ $CLEAN == 1 ]];
then
	echo
	echo "Cleaning up..."
	echo

	rm -rf $INDIR
fi
EOF2
chmod +x $SMSJOBSUM
# =============================================================================




echo "Commands to submit the jobs to Iceberg ($QUEUE queue):"
echo
echo "qsub $SMSJOB"
echo
echo "qsub $SMSJOBSUM"
echo
