#!/bin/bash
# (c) 2020 - Victor Soria-Carrasco
# victor.soria.carrasco@gmail.com
# Last modified: 23/04/2018 23:01:10

# Description:
# Extracts the SNVs for a particular species or population and
# create a bcf file for each phenotyped trait, filtering out by
# number of samples phenotyped. It takes as inputs a file with
# the samples and phenotypes and a raw bcf file with all the SNVs.
# The phenotype file is expected to have 9 or more columns separated by tabs:
# sample id, species, location, host, genotype (=if sequence data is
# available or not), number of parsed reads, number of mapped reads, sex,
# and as many traits as wanted in the following columns. The outputs are
# a separate bcf file for each trait.


BCFTOOLS='/usr/local/extras/Genomics/apps/bcftools/1.3.1/bin/bcftools'

PHENOS="phenotypes.dsv"
BCFRAW="variants.raw.bcf"
OUTDIR="./subsets"

mkdir -p $OUTDIR >& /dev/null

# general filters
MAF='0.01'
COV='0.5'
MINDP=10
MAXDPMULT=10 # multiplier relative to number of samples in dataset
QS=20
MINMAPREADS=100000 # minimum number of mapped reads to include an individual

# Expected first 8 columns are: id species location host genotype n_parsed n_mapped sex
TRAITS=($(head -n1 $PHENOS | cut  -f8-))

# Get a bcf file for each trait
for TI in $(seq 1 $((${#TRAITS[*]}-1)));
do
	TRAIT=${TRAITS[$TI]}
	
	# Include only samples with genotypes (more than $MINMAPREADS mapped reads) 
	# and phenotyped for this particular trait
	SAMPLES=$(tail -n+2 $PHENOS | 
			  awk '{if ($7 != "NA" &&
			            $7>='$MINMAPREADS' && 
						$'$(($TI+8))' != "NA") print $1 }' |
			  perl -pe 's/\n/\,/g' |
			  perl -pe 's/\,$//g')
	
	# Create phenotype file
	tail -n+2 $PHENOS | \
	awk '{if ($7 != "NA" &&
	          $7>='$MINMAPREADS' && 
			  $'$(($TI+8))' != "NA") print $'$(($TI+8))' }' \
	> $OUTDIR/$POP.$TRAIT.pheno
	

	NSAMPLES=$(echo $SAMPLES | awk -F, '{print NF}')

	MINNSAMPLES=$(perl -e 'print int('$NSAMPLES'*'$COV');')
	if [[ $MINNSAMPLES -gt $MINDP ]];
	then
		MINDP=$MINNSAMPLES
	fi

	echo ${#TRAITS[*]} - $TRAIT - $TI - $(($TI+8)) - $NSAMPLES

	MAXDP=$(($NSAMPLES*$MAXDPMULT))

	echo MAXDP $MAXDP

	OUT=$OUTDIR/$POP.$TRAIT.bial.noindel.qs"$QS".cov"$(perl -e 'print ('$COV'*100);')".mdp"$MINDP"Mdp"$MAXDP".maf"$MAF".bcf

	echo $OUT

	EXCLUDE='MAF<'$MAF' | QUAL<'$QS' | TYPE!="snp" | N_ALT>1 | (AN/2)<('$NSAMPLES'*'$COV') | SUM(FMT/DP)<'$MINDP' | SUM(FMT/DP)>'$MAXDP' | AC/AN=1'

	$BCFTOOLS view -s "$SAMPLES" -O u $BCFRAW | \
	$BCFTOOLS filter -S . -O u -e 'FMT/DP=0' | \
	$BCFTOOLS filter -e "$EXCLUDE" -O b -o $OUT

	$BCFTOOLS index $OUT
done

