#!/usr/bin/env perl

# (c) Romain Villoutreix
# r.villoutreix@sheffield.ac.uk
# Last modified: 09/04/2018

# Description:
# This script excute automaticaly the analysis for GenABEL, with and without control for population structure. The function is also performing an analysis restricting the control for population structure to LG different from LG8 and LG14.

#LOG
#09-04-2018 slight change in graphics display: y-axis log10 as a y-axis label to make things clearer. What was actually displayed is the log10, so this is just a cosmetic change

use warnings;
use strict;
use Getopt::Long;
use File::Basename;
use File::Path qw(make_path);
use File::Spec;
use Statistics::R;
use POSIX qw(strftime);

my $version='2.1.0-:-2018.04.09';

#Default outfiles
my $outdir='./';
my $traittype="guess";
my $maf=0.01;
my $bonferroni=0;
my $HWE=0.01;
my $naxisibs=3;
my $cumlenfile='/usr/local/extras/Genomics/scripts/gwas/useful_files/tcristinae_draft_1.3b2.scalen.scaf-1.cumlen.dsv';

&author;
my ($phenofile, $genofile, $trait);
GetOptions( 
    'p|P=s'  => \$phenofile,
    'g|G=s' => \$genofile,
	's|S=s' => \$cumlenfile,
    't|T=s' => \$trait,
    'y|Y=s' => \$traittype,
	'm|M=f' => \$maf,
	'b|B=f' => \$bonferroni,
	'hw|HW=f' => \$HWE,
	'n|N=i' => \$naxisibs,
    'o|O=s'  => \$outdir,
    'h|help' => \&usage 
);

&usage if (!defined($phenofile) || !defined($genofile) || !defined($trait));

#pheno file
if (! -e $phenofile){
die ("\nCan't access pheno file: $phenofile\n\n");
}

#geno file
if (! -e $genofile){
die ("\nCan't access geno file: $genofile\n\n");
}

#outdir
if (! -e $outdir){
system("mkdir $outdir");
}

#check for values of traittype
if ($traittype=~/gaussian|binomial|guess/){
print "\ntrait type is $traittype\n\n";} else {
die ("\nwrong type of trait type: choose between gaussian, binomial or guess\n\n");
}

$phenofile=File::Spec->rel2abs($phenofile);
$genofile=File::Spec->rel2abs($genofile);
$outdir=File::Spec->rel2abs($outdir);
my $inbase=basename($phenofile);
$inbase=~ s/\.pheno//g;
my $outbase="$outdir/".basename($phenofile);
$outbase=~ s/\.pheno//g;

print "maf is $maf\n\n";

print "HWE control with p.value of HWE <= $HWE (0 means no control!)\n\n";

print "Necessary files found :) \n\n\  Running GenABEL and ploting results \n\n"; 

print "  phenofile used: $phenofile\n\n";

print "  genofile used: $genofile\n\n";

print "  scaf-1 cumulative length file used: $cumlenfile\n\n";

# R code - put whatever you want in here
# --------------------------------------------------------------------------------------------

my $R = Statistics::R->new();

$R->run(qq`

setwd("$outdir");

library(GenABEL);

#### define useful variables
bonferroni <- $bonferroni;

#### load and explore data

dat <- load.gwaa.data(phenofile="$phenofile", genofile="$genofile", force=TRUE, makemap=FALSE, sort=TRUE, id = "id");

#### load scaf-1 cummulative length file
scaflen <- read.table("$cumlenfile", header=TRUE, sep="\t", na.string="NA");

cat(paste("maf:$maf\nHWE:$HWE\nninds before filtering:", nids(dat), "\nnSNPs before filtering:", nsnps(dat),"\n", sep=""), file="$outbase\_runs_infos.txt", append=FALSE);

#### QC

# QC round 1 - #no control for hardy weinberg here (option p.value=0), control for maf of given value, defaults for SNP and individual call rates (0.95 % of values), ibs threshold of 0.95% similarity (based on 2000SNPs)
# Also control for ind with extreme heterozygosity
# and those with IBS >= 0.95
qc1 <- check.marker(dat, p.level=0, maf = $maf); 
data1 <- dat[qc1\$idok, qc1\$snpok]; # data set containing only individuals and snps passing qc1

#attach(data1\@phdata);

# QC round 2 - repeat QC 1 step and add a check for HWE
qc2 <- check.marker(data1, p.level=$HWE, maf = $maf); # remove snps that are out of HWE at p defined by user
data1 <- data1[qc2\$idok, qc2\$snpok];

# prep data for GWAS: calculating kinship matrix, replacing diagonal for egscore funtion purposes, see ?egscore 
data1.gkin <- ibs(data1[, data1\@gtdata\@chromosome != "X"], weight="freq"); 
diag(data1.gkin) <- hom(data1[,autosomal(data1)])\$Var;
# second kinship for analysis with control excluding LG8 and LGNA
data1.gkin.excl_8_NA <- ibs(data1[, ((data1\@gtdata\@chromosome != "X") & (data1\@gtdata\@chromosome != 8) & (data1\@gtdata\@chromosome != 14))], weight="freq");
diag(data1.gkin.excl_8_NA) <- hom(data1[,autosomal(data1)])\$Var;

#compute bonferroni correction only if a threshold not already given. Export set-up values in a file
if(bonferroni==0){bonferroni <- (0.05/nsnps(data1))}

cat(paste("ninds after filtering:", nids(data1), "\nnSNPs after filtering:", nsnps(data1),"\nbonferroni:", bonferroni, "\nnumber of kinship PCA axis used as covariate:$naxisibs",  sep=""), file="$outbase\_runs_infos.txt", append=TRUE);

##########################################################
# single_SNP GWA WITHOUT correction for population structure#
#############################################################

#GWA
data1.wok <- qtscore($trait, data1, trait.type="$traittype");
#control for sex not necessary
#data1.wok <- qtscore($trait~as.factor(sex), data=data1, trait.type="$traittype");

# summary table of SNPs and p's:
results <- data1.wok\@results;

results\$lg <- "NA";
results\$lg <- as.character(results\$lg);
results\$lg <- substr(sapply(strsplit(sapply(strsplit(row.names(results), ":"), "[[", 1 ), "\_"), "[[", 1), 3, nchar(sapply(strsplit(sapply(strsplit(row.names(results), ":"), "[[", 1 ), "\_"), "[[", 1)));
results\$lg[results\$lg=="NA"] <- 14;
results\$lg <- as.numeric(results\$lg);

results\$ord <- "NA";
results\$ord <- as.character(results\$ord);
results\$ord <- substr(sapply(strsplit(sapply(strsplit(row.names(results), ":"), "[[", 1 ), "\_"), "[[", 2), 4, nchar(sapply(strsplit(sapply(strsplit(row.names(results), ":"), "[[", 1 ), "\_"), "[[", 2)));
results\$ord[results\$ord=="NA"] <- 1;
results\$ord <- as.numeric(results\$ord);

results\$scaf <- "NA";
results\$scaf <- as.character(results\$scaf);
results\$scaf <- substr(sapply(strsplit(sapply(strsplit(row.names(results), ":"), "[[", 1 ), "\_"), "[[", 3), 5, nchar(sapply(strsplit(sapply(strsplit(row.names(results), ":"), "[[", 1 ), "\_"), "[[", 3)));
results\$scaf <- as.numeric(results\$scaf);

results\$pos <- "NA";
results\$pos <- as.character(results\$pos);
results\$pos <- sapply(strsplit(row.names(results), ":"), "[[", 2 );
results\$pos <- as.numeric(results\$pos);

results <- results[order(results\$lg, results\$ord, results\$scaf, results\$pos),];
results\$ass <- ifelse(results\$P1df <= bonferroni, "yes", "no");
results\$rel.pos <- seq(1, dim(results)[1], 1);
results\$neg.log.p <- -log(results\$P1df, base = 10);

results\$genome_pos_w_Ns <- NA;

for(i in c(unique(results\$scaf))){
results\$genome_pos_w_Ns[results\$scaf==i] <- results\$pos[results\$scaf==i] + scaflen\$cumlength_w_Ns[scaflen\$scafnumber==i];
}

write.table(results, file="$outbase\_sparse_without_correction_for_pop_structure.txt", row.names=FALSE, col.names=TRUE, dec=".", na="NA", eol="\n", sep="\t");

#### plot, romain's quick and dirty version :

# prepare useful table for plot

indextable <- aggregate(results[,match("genome_pos_w_Ns", colnames(results))], list(results\$lg), mean);

jpeg("$outbase\_without_correction_for_pop_structure.jpg", width=21, height=10, units="cm", pointsize=12, res=600);

results\$col <- "grey80";
results[which(results\$lg \%in\% c(1,3,5,7,9,11,13)),]\$col <- "gray40";
results[which(results\$lg \%in\% c(2,4,6,8,10,12)),]\$col <- "gray0";

plot(NULL, NULL, xlab = "linkage group", ylab = "-log10(P)", ylim = c(0, max(-log(results\$P1df, base = 10), na.rm=TRUE)),  xlim = c(0,(max(scaflen\$cumlength_w_Ns)+scaflen\$length_w_Ns[nrow(scaflen)])), bty = "n", xaxt = "n");

points(results\$genome_pos_w_Ns, -log(results\$P1df, base = 10), col = c(results\$col), cex = 0.5);
points(results[which(results\$ass == "yes"),]\$genome_pos_w_Ns, -log(results[which(results\$ass == "yes"),]\$P1df, base = 10), col="red", cex = 0.5);
text(x=c(indextable[,2]), y = rep((0-(max(results\$neg.log.p, na.rm=TRUE)-0)*0.10), nrow(indextable)), labels = c(indextable[,1]), xpd=TRUE, cex=0.8);

dev.off();



##########################################################
## single_SNP GWA with correction for population structure#
## using the method of Price et al. (using PCs of the genomic kinship matrix) here implemented in the egscore function (you just have to provide the kinship matrix)
##############################################################################

#GWA
data1.pca <- egscore($trait, data1, kin=data1.gkin, propPs=0.95, clambda=FALSE, naxes=$naxisibs);
#control for sex not necessary
#data1.pca <- egscore($trait~as.factor(sex), data=data1, kin=data1.gkin, propPs=0.95, clambda=FALSE, naxes=$naxisibs);

## summary table of SNPs and p's:
results <- data1.pca\@results;


results\$lg <- "NA";
results\$lg <- as.character(results\$lg);
results\$lg <- substr(sapply(strsplit(sapply(strsplit(row.names(results), ":"), "[[", 1 ), "\_"), "[[", 1), 3, nchar(sapply(strsplit(sapply(strsplit(row.names(results), ":"), "[[", 1 ), "\_"), "[[", 1)));
results\$lg[results\$lg=="NA"] <- 14;
results\$lg <- as.numeric(results\$lg);

results\$ord <- "NA";
results\$ord <- as.character(results\$ord);
results\$ord <- substr(sapply(strsplit(sapply(strsplit(row.names(results), ":"), "[[", 1 ), "\_"), "[[", 2), 4, nchar(sapply(strsplit(sapply(strsplit(row.names(results), ":"), "[[", 1 ), "\_"), "[[", 2)));
results\$ord[results\$ord=="NA"] <- 1;
results\$ord <- as.numeric(results\$ord);

results\$scaf <- "NA";
results\$scaf <- as.character(results\$scaf);
results\$scaf <- substr(sapply(strsplit(sapply(strsplit(row.names(results), ":"), "[[", 1 ), "\_"), "[[", 3), 5, nchar(sapply(strsplit(sapply(strsplit(row.names(results), ":"), "[[", 1 ), "\_"), "[[", 3)));
results\$scaf <- as.numeric(results\$scaf);

results\$pos <- "NA";
results\$pos <- as.character(results\$pos);
results\$pos <- sapply(strsplit(row.names(results), ":"), "[[", 2 );
results\$pos <- as.numeric(results\$pos);

results <- results[order(results\$lg, results\$ord, results\$scaf, results\$pos),];
results\$ass <- ifelse(results\$Pc1df <= bonferroni, "yes", "no");
results\$rel.pos <- seq(1, dim(results)[1], 1);
results\$neg.log.p <- -log(results\$Pc1df, base = 10);

results\$genome_pos_w_Ns <- NA;
 
for(i in c(unique(results\$scaf))){
results\$genome_pos_w_Ns[results\$scaf==i] <- results\$pos[results\$scaf==i] + scaflen\$cumlength_w_Ns[scaflen\$scafnumber==i];
}

write.table(results, file="$outbase\_sparse_with_correction_for_pop_structure.txt", row.names=FALSE, col.names=TRUE, dec=".", na="NA", eol="\n", sep="\t");

##### plot, romain's quick and dirty version :

# prepare useful table for plot 

indextable <- aggregate(results[,match("genome_pos_w_Ns", colnames(results))], list(results\$lg), mean);

jpeg("$outbase\_with_correction_for_pop_structure.jpg", width=21, height=10, units="cm", pointsize=12, res=600);

results\$col <- "grey80";
results[which(results\$lg \%in\% c(1,3,5,7,9,11,13)),]\$col <- "gray40";
results[which(results\$lg \%in\% c(2,4,6,8,10,12)),]\$col <- "gray0";

plot(NULL, NULL, xlab = "linkage group", ylab = "-log10(P)", ylim = c(0, max(-log(results\$Pc1df, base = 10), na.rm=TRUE)),  xlim = c(0,(max(scaflen\$cumlength_w_Ns)+scaflen\$length_w_Ns[nrow(scaflen)])), bty = "n", xaxt = "n");

points(results\$genome_pos_w_Ns, -log(results\$Pc1df, base = 10), col = c(results\$col), cex = 0.5);
points(results[which(results\$ass == "yes"),]\$genome_pos_w_Ns, -log(results[which(results\$ass == "yes"),]\$Pc1df, base = 10), col="red", cex = 0.5);
text(x=c(indextable[,2]), y = rep((0-(max(results\$neg.log.p, na.rm=TRUE)-0)*0.10), nrow(indextable)), labels = c(indextable[,1]), xpd=TRUE, cex=0.8);

dev.off();

##########################################################
### single_SNP GWA with correction for population structure but excluding LG8 and LGNA from kinship matrix calculation
### using the method of Price et al. (using PCs of the genomic kinship matrix) here implemented in the egscore function (you just have to provide the kinship matrix)
###############################################################################

#GWA
data1.pca.excl8_NA <- egscore($trait, data1, kin=data1.gkin.excl_8_NA, propPs=0.95, clambda=FALSE, naxes=$naxisibs);
#control for sex not necessary
#data1.pca.excl8_NA <- egscore($trait~as.factor(sex), data=data1, kin=data1.gkin.excl_8_NA, propPs=0.95, clambda=FALSE, naxes=$naxisibs);

### summary table of SNPs and p's:
results <- data1.pca.excl8_NA\@results;


results\$lg <- "NA";
results\$lg <- as.character(results\$lg);
results\$lg <- substr(sapply(strsplit(sapply(strsplit(row.names(results), ":"), "[[", 1 ), "\_"), "[[", 1), 3, nchar(sapply(strsplit(sapply(strsplit(row.names(results), ":"), "[[", 1 ), "\_"), "[[", 1)));
results\$lg[results\$lg=="NA"] <- 14;
results\$lg <- as.numeric(results\$lg);

results\$ord <- "NA";
results\$ord <- as.character(results\$ord);
results\$ord <- substr(sapply(strsplit(sapply(strsplit(row.names(results), ":"), "[[", 1 ), "\_"), "[[", 2), 4, nchar(sapply(strsplit(sapply(strsplit(row.names(results), ":"), "[[", 1 ), "\_"), "[[", 2)));
results\$ord[results\$ord=="NA"] <- 1;
results\$ord <- as.numeric(results\$ord);

results\$scaf <- "NA";
results\$scaf <- as.character(results\$scaf);
results\$scaf <- substr(sapply(strsplit(sapply(strsplit(row.names(results), ":"), "[[", 1 ), "\_"), "[[", 3), 5, nchar(sapply(strsplit(sapply(strsplit(row.names(results), ":"), "[[", 1 ), "\_"), "[[", 3)));
results\$scaf <- as.numeric(results\$scaf);

results\$pos <- "NA";
results\$pos <- as.character(results\$pos);
results\$pos <- sapply(strsplit(row.names(results), ":"), "[[", 2 );
results\$pos <- as.numeric(results\$pos);

results <- results[order(results\$lg, results\$ord, results\$scaf, results\$pos),];
results\$ass <- ifelse(results\$Pc1df <= bonferroni, "yes", "no");
results\$rel.pos <- seq(1, dim(results)[1], 1);
results\$neg.log.p <- -log(results\$Pc1df, base = 10);

results\$genome_pos_w_Ns <- NA;
 
for(i in c(unique(results\$scaf))){
results\$genome_pos_w_Ns[results\$scaf==i] <- results\$pos[results\$scaf==i] + scaflen\$cumlength_w_Ns[scaflen\$scafnumber==i];
}

write.table(results, file="$outbase\_sparse_with_correction_LG8_NA_excluded_for_pop_structure.txt", row.names=FALSE, col.names=TRUE, dec=".", na="NA", eol="\n", sep="\t");

##### plot, romain's quick and dirty version :

# prepare useful table for plot 

indextable <- aggregate(results[,match("genome_pos_w_Ns", colnames(results))], list(results\$lg), mean);

jpeg("$outbase\_with_correction_LG8_NA_excluded_for_pop_structure.jpg", width=21, height=10, units="cm", pointsize=12, res=600);

results\$col <- "grey80";
results[which(results\$lg \%in\% c(1,3,5,7,9,11,13)),]\$col <- "gray40";
results[which(results\$lg \%in\% c(2,4,6,8,10,12)),]\$col <- "gray0";

plot(NULL, NULL, xlab = "linkage group", ylab = "-log10(P)", ylim = c(0, max(-log(results\$Pc1df, base = 10), na.rm=TRUE)),  xlim = c(0,(max(scaflen\$cumlength_w_Ns)+scaflen\$length_w_Ns[nrow(scaflen)])), bty = "n", xaxt = "n");

points(results\$genome_pos_w_Ns, -log(results\$Pc1df, base = 10), col = c(results\$col), cex = 0.5);
points(results[which(results\$ass == "yes"),]\$genome_pos_w_Ns, -log(results[which(results\$ass == "yes"),]\$Pc1df, base = 10), col="red", cex = 0.5);
text(x=c(indextable[,2]), y = rep((0-(max(results\$neg.log.p, na.rm=TRUE)-0)*0.10), nrow(indextable)), labels = c(indextable[,1]), xpd=TRUE, cex=0.8);

dev.off();


`);


$R->stop();

print " Done!  Output files and graphs saved in $outdir \n\n  Bye \n\n";

# --------------------------------------------------------------------------------------------#


# ==============================================================================
# ==============================================================================
# ============================== SUBROUTINES ===================================
# ==============================================================================
# ==============================================================================

# Show copyright
# ==============================================================================
sub author{
    print "\n";
    print "#########################################\n";
    print "  ".basename($0)."\n";
	print "  version $version\n";
    print "  (c) Romain Villoutreix            \n";
    print "  r.villoutreix\@sheffield.ac.uk      \n";
    print "#########################################\n";
	print "\n";
}
# ==============================================================================

# Show usage
# ==============================================================================
sub usage{
    print "\n";
	print "  Usage:\n";
    print "    ".basename($0)."\n";
	print "      -p <input file (.pheno file)>\n";
	print "      -g <input file (.raw file)>\n";
	print "      -s <scaffold -1 cummulative length file from compute_scaf-1_cummulative_length.pl function, default:$cumlenfile>\n";
	print "      -t <trait name>\n";
	print "      -y <trait type when not accounting for population structure (gaussian or binomial or guess) - optional - default: guess>\n";
	print "      -m <minor allelic frequency cut-off (0.01 for 1\%) - optional - default: 0.01>\n";
	print "      -b <bonferroni correction threshold (0.01 for 1\%) - optional - default: 0.05/number of tested SNPs>\n";
	print "      -hw <hardy-weinberg equilibrium threshold (0.01 for p.value of 0.01) - optional - default: 0.01 - put 0 for no control>\n";
	print "      -n <number of kinship pca axis used as covariate - optional - default: 3>\n";
	print "      -o <output directory - optional defaults to \$PWD>\n";
	print "      -h <show this help>\n";
    print "\n";
    exit;
}
# ==============================================================================
