#!/usr/bin/env perl

# (c) Romain Villoutreix
# romain.villoutreix@gmail.com
# Last modified: 24/10/2016

# Description:
# This script plots the sparse values (p-values along the genome) resulting from the GenABEL_analysis.pl function


use warnings;
use strict;
use Getopt::Long;
use File::Basename;
use File::Path qw(make_path);
use File::Spec;
use Statistics::R;
use POSIX qw(strftime);
use List::Util 1.33 'any';

my $version='2.0.0-2016.10.24';

#Default graphic variables
my $y=0;

#default LG=8
my $interstLG=8;

#default cumlengthfile
my $cumlenfile='/usr/local/extras/Genomics/db/genomes/timema_cristinae/tcristinae_draft_1.2b.scalen.scaf-1.cumlen.dsv';

#Default outfiles
my $outdir='output';
&author;
my ($infile);
GetOptions( 
    'i|I=s'  => \$infile,
    'l|L=i' => \$interstLG,
	's|S=s' => \$cumlenfile,
    'o|O=s'  => \$outdir,
	'y|Y=f' => \$y,
    'h|help' => \&usage 
);

&usage if (!defined($infile) || !defined($outdir));

#Input file does exist ?
if (! -e $infile){
die ("\nCan't access input file: $infile\n\n");
}

# Output directory does exist ?
if (! -e $outdir){
eval {make_path($outdir)}
or die ("\nError creating output directory: $outdir\n\n");
}

#verification that LG given exist
my @LGnumber = (1,2,3,4,5,6,7,8,9,10,11,12,13,14); 
my $match_found = any { /$interstLG/ } @LGnumber;

if($match_found==0){
die ("\nLG specified not good: must be an integer between 1-14\n\n");
}

#some specifications of path
$infile=File::Spec->rel2abs($infile);
$outdir=File::Spec->rel2abs($outdir);
my $inbase=basename($infile);
$inbase=~ s/\.txt//g;
my $outbase="$outdir/".basename($infile);
$outbase=~ s/\.txt//g;

print "Necessary files found :) \n\n\  Making graphics ! \n\n"; 

print "  sparse file used: $infile\n\n";

print "  LG plotted: $interstLG\n\n";

print "  scaf-1 cummulative lenght file used: $cumlenfile\n\n";


# R code - put whatever you want in here
# --------------------------------------------------------------------------------------------
   my $R = Statistics::R->new();
     
$R->run(qq`

setwd("$outdir");

#defining graphic variables

	y <- $y;

# opening sparse file and cumlenfile

	sparse <- read.table(file="$infile", sep="\t", dec=".", header=TRUE);

	scaflen <- read.table("$cumlenfile", header=TRUE, sep="\t", na.string="NA");

#change with the other function ! define locus on no LG as associated with lg 14, and scaffold order 0 

	sparse\$lg[is.na(sparse\$lg)] <- 14;
	sparse\$ord[sparse\$lg==14] <- 0;
	sparsemapped <- sparse;

#select the LG of interest

	sparsemapped <- sparsemapped[sparsemapped\$lg==$interstLG,];

# plot the -log p.values along the interest LG taking into account physical order

	sparsemapped\$colour <- "NA";
	for (i in 1:length(unique(sparsemapped\$scaf))) {
	if(i%%2==0){sparsemapped\$colour[sparsemapped\$scaf==unique(sparsemapped\$scaf)[i]] <- "gray40";}
	if(i%%2!=0){sparsemapped\$colour[sparsemapped\$scaf==unique(sparsemapped\$scaf)[i]] <- "gray0";}
	}

# prepare useful table for plot

	indextable <- aggregate(sparsemapped[,match("genome_pos_w_Ns", colnames(sparsemapped))], list(sparsemapped\$scaf), mean);

	jpeg(filename="$outbase\_pvalues_along_LG_$interstLG\.jpg", width=21, height=10, units="cm", pointsize=12, res=600);

	if(y==0){
	plot(NULL, NULL, xlab = "linkage group $interstLG", ylab = "-log(P)", ylim = c(0, max(sparsemapped\$neg.log.p, na.rm=TRUE)),  xlim = c(min(scaflen\$cumlength_w_Ns[scaflen\$lg==$interstLG]), (max(scaflen\$cumlength_w_Ns[scaflen\$lg==$interstLG]) + tail(scaflen\$length_w_Ns[scaflen\$lg==$interstLG], 1))), bty = "n", xaxt = "n");
	points(sparsemapped\$genome_pos_w_Ns, sparsemapped\$neg.log.p, col = c(sparsemapped\$colour), cex = 0.5);
	points(sparsemapped[which(sparsemapped\$ass == "yes"),]\$genome_pos_w_Ns, sparsemapped[which(sparsemapped\$ass == "yes"),]\$neg.log.p, col="red", cex = 0.5);
	text(x=c(indextable[,2]), y = rep((0-(max(sparsemapped\$neg.log.p, na.rm=TRUE)-0)*0.10), nrow(indextable)), labels = c(indextable[,1]), xpd=TRUE, cex=0.45, srt=90);
	}
	
	if(y!=0){
	plot(NULL, NULL, xlab = "linkage group $interstLG", ylab = "-log(P)", ylim = c(0, y),  xlim = c(min(scaflen\$cumlength_w_Ns[scaflen\$lg==$interstLG]), (max(scaflen\$cumlength_w_Ns[scaflen\$lg==$interstLG]) + tail(scaflen\$length_w_Ns[scaflen\$lg==$interstLG], 1))), bty = "n", xaxt = "n");
	points(sparsemapped\$snpindex, sparsemapped\$neg.log.p, col = c(sparsemapped\$colour), cex = 0.5);
	points(sparsemapped[which(sparsemapped\$ass == "yes"),]\$genome_pos_w_Ns, sparsemapped[which(sparsemapped\$ass == "yes"),]\$neg.log.p, col="red", cex = 0.5);
	text(x=c(indextable[,2]), y = rep((0-(y-0)*0.10), nrow(indextable)), labels = c(indextable[,1]), xpd=TRUE, cex=0.45, srt=90);
	}

	dev.off();`);

$R->stop();

print " Done!  Graphs saved in $outdir \n\n  Bye \n\n";

# --------------------------------------------------------------------------------------------#


# ==============================================================================
# ==============================================================================
# ============================== SUBROUTINES ===================================
# ==============================================================================
# ==============================================================================

# Show copyright
# ==============================================================================
sub author{
    print "\n";
    print "#########################################\n";
    print "  ".basename($0)."\n";
	print "  version $version\n";
    print "  (c) Romain Villoutreix            \n";
    print "  romain.villoutreix\@gmail.com      \n";
    print "#########################################\n";
	print "\n";
}
# ==============================================================================

# Show usage
# ==============================================================================
sub usage{
    print "\n";
	print "  Usage:\n";
    print "    ".basename($0)."\n";
	print "      -i <input file (sparse file output from GenABEL_analysis.pl)>\n";
	print "      -l <Linkage group of interest (designed for T. cristinae: 1-14; 14 beeing scaffold associated with no Linkage Group) / default=LG8>\n";
	print "      -s <scaffold -1 cummulative length file from compute_scaf-1_cummulative_length.pl function, default: '$cumlenfile'>\n";
	print "      -o <output directory - optional defaults to output/>\n";
	print "      -y <max y display value for pvalue along LG of interest graphic / default = automatic value>\n";
	print "      -h <show this help>\n";
    print "\n";
    exit;
}
# ==============================================================================
