#!/usr/bin/env perl

# (c) Romain Villoutreix
# romain.villoutreix@gmail.com
# Last modified: 05/10/2016

# Description:
# This script plots the sparse values (p-values along the genome) resulting from the GenABEL_analysis.pl function on a zoomed portion of a scaffold

## Beware, the function does not check if the values exist in your genome ##

use warnings;
use strict;
use Getopt::Long;
use File::Basename;
use File::Path qw(make_path);
use File::Spec;
use Statistics::R;
use POSIX qw(strftime);
use List::Util 1.33 'any';

my $version='1.0.0-2017.11.08';

#Default graphic variables
my $y=0;

#default scaffold=128, start=5Mbp, end 6Mbp
my $interstscaf=128;;
my $start=50000000;
my $end=6000000;

#Default outfiles
my $outdir='output';
&author;
my ($infile);
GetOptions( 
    'i|I=s'  => \$infile,
    's|S=f' => \$interstscaf,
	'st=i' => \$start,
	'en=i' => \$end,
    'o|O=s'  => \$outdir,
	'y=f' => \$y,
    'h|help' => \&usage 
);

&usage if (!defined($infile) || !defined($outdir));

#Input file does exist ?
if (! -e $infile){
die ("\nCan't access input file: $infile\n\n");
}

# Output directory does exist ?
if (! -e $outdir){
eval {make_path($outdir)}
or die ("\nError creating output directory: $outdir\n\n");
}

#verification that given scaffold exist
#my @scafnumber = (1,2,3,4,5,6,7,8,9,10,11,12,13,14); 
#my $match_found = any { /$interstscaf/ } @scafnumber;

#if($match_found==0){
#die ("\nScaffold specified not good: not contained in the sparse matrix\n\n");
#}

#some specifications of path
$infile=File::Spec->rel2abs($infile);
$outdir=File::Spec->rel2abs($outdir);
my $inbase=basename($infile);
$inbase=~ s/\.txt//g;
my $outbase="$outdir/".basename($infile);
$outbase=~ s/\.txt//g;

print "Necessary files found :) \n\n\  Making graphics ! \n\n"; 

print "  sparse file used: $infile\n\n";

print "  scaffold of interest: $interstscaf start: $start end $end\n\n";

# R code - put whatever you want in here
# --------------------------------------------------------------------------------------------
   my $R = Statistics::R->new();
     
$R->run(qq`

setwd("$outdir");

#defining graphic variables

	y <- $y;

# opening sparse file

	sparse <- read.table(file="$infile", sep="\t", dec=".", header=TRUE);

#select the LG of interest
	
	sparsemapped <- sparse[sparse\$scaf==$interstscaf,];
	sparsemapped <- sparsemapped[((sparsemapped\$pos>=$start) & (sparsemapped\$pos<=$end)),];

# plot the -log p.values along the interest scaffold taking physical distance into account

	sparsemapped\$colour <- "grey0";

	jpeg(filename="$outbase\_pvalues_along_scaf_$interstscaf\_$start\_$end.jpg", width=21, height=10, units="cm", pointsize=12, res=600);

	if(y==0){
	plot(NULL, NULL, xlab = "scaffold number $interstscaf", ylab = "-log(P)", ylim = c(0, max(sparsemapped\$neg.log.p, na.rm=TRUE)),  xlim = c($start, $end), bty = "n", xaxt = "n");
	points(sparsemapped\$pos, sparsemapped\$neg.log.p, col = c(sparsemapped\$colour), cex = 0.5);
	points(sparsemapped[which(sparsemapped\$ass == "yes"),]\$pos, sparsemapped[which(sparsemapped\$ass == "yes"),]\$neg.log.p, col="red", cex = 0.5);
	axis(1, seq($start, $end, 50000));
	}
	
	if(y!=0){
	plot(NULL, NULL, xlab = "linkage group $interstscaf", ylab = "-log(P)", ylim = c(0, y),  xlim = c($start, $end), bty = "n", xaxt = "n");
	points(sparsemapped\$pos, sparsemapped\$neg.log.p, col = c(sparsemapped\$colour), cex = 0.5);
	points(sparsemapped[which(sparsemapped\$ass == "yes"),]\$pos, sparsemapped[which(sparsemapped\$ass == "yes"),]\$neg.log.p, col="red", cex = 0.5);
	axis(1, seq($start, $end, 50000));
	}

	dev.off();`);

$R->stop();

print " Done!  Graphs saved in $outdir \n\n  Bye \n\n";

# --------------------------------------------------------------------------------------------#


# ==============================================================================
# ==============================================================================
# ============================== SUBROUTINES ===================================
# ==============================================================================
# ==============================================================================

# Show copyright
# ==============================================================================
sub author{
    print "\n";
    print "#########################################\n";
    print "  ".basename($0)."\n";
	print "  version $version\n";
    print "  (c) Romain Villoutreix            \n";
    print "  romain.villoutreix\@gmail.com      \n";
    print "#########################################\n";
	print "\n";
}
# ==============================================================================

# Show usage
# ==============================================================================
sub usage{
    print "\n";
	print "  Usage:\n";
    print "    ".basename($0)."\n";
	print "      -i <input file (sparse file output from GenABEL_analysis.pl)>\n";
	print "      -s <scaffold of interest (designed for T. cristinae: but no check of good value given yet, has to be a number) / default=$interstscaf>\n";
	print "      -st <start position on scaffold (designed for T. cristinae: but no check of good value given yet, has to be a number) / default=$start>\n";
	print "      -en <end position on scaffold (designed for T. cristinae: but no check of good value given yet, has to be a number) / default=$end>\n";
	print "      -o <output directory - optional defaults to output/>\n";
	print "      -y <max y display value for pvalue along LG of interest graphic / default = automatic value>\n";
	print "      -h <show this help>\n";
    print "\n";
    exit;
}
# ==============================================================================
