#!/usr/bin/env perl

# (c) Romain Villoutreix
# r.villoutreix@sheffield.ac.uk
# Last modified: 18/10/2016

# Description:
# This script compute the scaf-1 cumulative file you'll need for GenABEL analysis.
# !!! This function was originally designed for Timema and set scaffold in lg NA as belonging to lg 14 by default (but you can change this) with an order of 1 i.e. they will be plotted by scaffold number !!!
# This function need a scaffolds/contigs named this way to work properly: lgX_ordX_scafx

use warnings;
use strict;
use Getopt::Long;
use File::Basename;
use File::Path qw(make_path);
use File::Spec;
use Statistics::R;
use POSIX qw(strftime);

my $version='1.3.0-2018.04.30';

#Default outfiles
my $outdir='./';
my $LGNA=14;
&author;
my ($scaflenfile);
GetOptions( 
    'i|I=s'  => \$scaflenfile,
    'o|O=s'  => \$outdir,
	'l|L=i' => \$LGNA,
    'h|help' => \&usage 
);

&usage if (!defined($scaflenfile));

#scaffold length file
if (! -e $scaflenfile){
die ("\nCan't access scaffold length file: $scaflenfile\n\n");
}

#outdir
if (! -e $outdir){
exec ("mkdir $outdir") or print STDERR "couldn't create outdir: $!";
}

$scaflenfile=File::Spec->rel2abs($scaflenfile);
$outdir=File::Spec->rel2abs($outdir);
my $inbase=basename($scaflenfile);
$inbase=~ s/\.dsv//g;
my $outbase="$outdir/".basename($scaflenfile);
$outbase=~ s/\.dsv//g;

print "Necessary files found :) \n\n\  Computing cumulative scaffold length\n\n"; 


#Opening of the input and output files
my @fields=();
my @subfield=();
my $lg=();
my $scafnumber=();
my $scaforder=();

open (OFH, ">$outbase\.tmp") or die "\nCan't open $outbase\.tmp file\n\n";
print OFH "lg\tscaforder\tscafnumber\tlength_w_Ns\tlength_wo_Ns\n";

open (IFH, $scaflenfile) or die "\nCan't open $scaflenfile file\n\n";
	while (<IFH>){
	#skip the first line
	next if $. == 1;
	chomp $_;
	@fields = split("\t",$_);
	@subfield = split("_", $fields[0]);
	$lg = substr $subfield[0], 2;
	$scaforder = substr $subfield[1], 3;
	$scafnumber = substr $subfield[2], 4;
										
	print OFH "$lg\t$scaforder\t$scafnumber\t$fields[1]\t$fields[2]\n";

}

#closing the files
close(IFH);
close(OFH);





# R code - sort scaffold and compute cumulative lengths
# --------------------------------------------------------------------------------------------

my $R = Statistics::R->new();

$R->run(qq`

#### load formatted file with scaffold length

data <- read.table("$outbase\.tmp", header=TRUE, sep="\t", na.string="NA");

data\$lg[is.na(data\$lg)] <- $LGNA;
data\$scaforder[is.na(data\$scaforder)] <- 1;

asI <- function(x){as.integer(x)};

datasorted <- data[order(asI(data\$lg), asI(data\$scaforder), asI(data\$scafnumber)),];

datasorted\$cumlength_w_Ns <- NA;
datasorted\$cumlength_wo_Ns <- NA;

datasorted\$cumlength_w_Ns[1] <- 0;
datasorted\$cumlength_wo_Ns[1] <- 0;


for(i in 2:nrow(datasorted)){
datasorted\$cumlength_w_Ns[i] <- datasorted\$cumlength_w_Ns[i-1] + datasorted\$length_w_Ns[i-1];
datasorted\$cumlength_wo_Ns[i] <- datasorted\$cumlength_wo_Ns[i-1] + datasorted\$length_wo_Ns[i-1];
}

write.table(datasorted, file="$outbase\.scaf\-1.cumlen.dsv", row.names=FALSE, col.names=TRUE, dec=".", na="NA", eol="\n", sep="\t", quote=FALSE);`);

$R->stop();

system("rm $outbase\.tmp");

print " Done!  scaffold-1 Cumulative length saved as $outbase\.scaf\-1.cumlen.dsv \n\n  Bye \n\n";

# --------------------------------------------------------------------------------------------#


# ==============================================================================
# ==============================================================================
# ============================== SUBROUTINES ===================================
# ==============================================================================
# ==============================================================================

# Show copyright
# ==============================================================================
sub author{
    print "\n";
    print "#########################################\n";
    print "  ".basename($0)."\n";
	print "  version $version\n";
    print "  (c) Romain Villoutreix            \n";
    print "  r.villoutreix\@sheffield.ac.uk      \n";
    print "#########################################\n";
	print "\n";
}
# ==============================================================================

# Show usage
# ==============================================================================
sub usage{
    print "\n";
	print "  Usage:\n";
    print "    ".basename($0)."\n";
	print "      -i <input file (.dsv file with scaffold lengths)>\n";
	print "      -o <output directory - optional defaults to \$WD>\n";
	print "      -l <linkage group number for LGNA scaffold/contigs - default set for Timema to LG $LGNA but change it for your haplotype chromosome number+1!>\n";
	print "      -h <show this help>\n";
    print "\n";
    exit;
}
# ==============================================================================
