#!/usr/bin/env perl
# original code by (c) Doro Lindtke 
# further modified by (c) Victor Soria-Carrasco
# victor.soria.carrasco@gmail.com
# Last modified: 20/07/2017 06:19:49

# Description:
# Using a genotype likelihood file as input, converts it to
# fastPHASE input file

## use .gl file to generate fastPHASE input file
## .gl file only containing one LG
##     (or extract lg8 here alternatively)
## usage: mkfastphaseinp.pl file.gl
## ---------------------------------------------

## makes a couple of (temporary) files too -
## the fastPHASE input file is *.inp

use warnings;
use strict;

my $thresh = 0.95; ## prob threshold to define allele as known
my $sepmissalleles = 1; ## code both alleles as missing (0) or only one (1)
                        ## (testing)

die "Usage: mkfastphaseinp.pl <glfile>\n" unless @ARGV == 1;

my $in = shift (@ARGV);
open (IN, $in) or die "Couldn't open input $in\n";

my $out = $in;
$out =~ s/gl$/inp/;
open (OUT, "> $out") or die "Couldn't open output $out\n";
my $tmpa1 = $in;
$tmpa1 =~ s/gl$/a1/;
open (TMPA1, "> $tmpa1") or die "Couldn't open output $tmpa1\n";
my $tmpa2 = $in;
$tmpa2 =~ s/gl$/a2/;
open (TMPA2, "> $tmpa2") or die "Couldn't open output $tmpa2\n";
my $snps = $in;
$snps =~ s/gl$/snps/;
open (SNPS, "> $snps") or die "Couldn't open output $snps\n";
my $map = $in;
$map =~ s/gl$/pos/ or die "Couldn't make file for SNP positions\n";

## read .gl file first line
my $row = <IN>;
chomp $row;
my @line = split(/\s+/,$row);
my $nind = $line[0];
## my $nloc = $line[1];
my $nloc = 0; ## alternatively, count loci below

## read ind IDs
$row = <IN>;
chomp $row;
my @indids = split(/\s+/,$row);

## read remaining lines for each locus
my $locid;
my @geno = ();
my @a1 = ();
my @a2 = ();
my @gl = (0, 0, 0);
my $sum;
my $i;
print SNPS "lg ord scaf pos\n";

while (<IN>){
    chomp;
    ## next if (!/^lg(\d+|NA)_/); ## this line has no genotype data
    # next if (!/^lg8_/); ## alternatively, check that only lg8 used
    next if (!/^lg/); ## discard non-lg lines
    @geno = split(/\s+/,$_);
    $locid = shift(@geno);
    $locid =~ s/^lg//;
    $locid =~ s/_ord/ /;
    $locid =~ s/_scaf/ /;
    $locid =~ s/:/ /;
    @a1 = ();
    @a2 = ();
    while (@geno){
	$sum = 0;
	for $i (0..2){ ## three genotype likelihoods for each individual
	    $gl[$i] = shift(@geno);
	    $gl[$i] = 10 ** ($gl[$i]/-10); ## take off phred
	    $sum += $gl[$i];
	}
	for $i (0..2){ ## normalize
	    $gl[$i] = $gl[$i]/$sum;
	}

	if ($gl[0] >= $thresh){
	    push(@a1, '0');
	    push(@a2, '0');
	}
	elsif ($gl[1] >= $thresh){
	    push(@a1, '0');
	    push(@a2, '1');
	}
	elsif ($gl[2] >= $thresh){
	    push(@a1, '1');
	    push(@a2, '1');
	}
	elsif ($sepmissalleles == 1){
	    if (($gl[0] + $gl[1]) >= $thresh){
		push(@a1, '0');
		push(@a2, '?');
	    }
	    elsif (($gl[1] + $gl[2]) >= $thresh){
		push(@a1, '?');
		push(@a2, '1');
	    }
	    else {
		push(@a1, '?');
		push(@a2, '?');
	    }
	}
	else {
	    push(@a1, '?');
	    push(@a2, '?');
	}
    }

    ## write temporary files for allele1 and allele2
    ## (inds are in columns)
    print TMPA1 "@a1\n";
    print TMPA2 "@a2\n";
    ## write temporary file with loci positions
    print SNPS "$locid\n";
    $nloc++;
}

close (IN);
close (TMPA1);
close (TMPA2);
close (SNPS);

## run R to compute recombination distances
my $recombdist=<<EOF;
bpcM<-1000000 ## bp per cM
scafdist<-2 ## arbitrary value for distance in cM between scaffolds
snps<-read.table("$snps",header=T)
if(length(unique(snps[,1]))!=1) stop("Only one LG allowed")
map<-numeric(dim(snps)[1])
curdist<-0
map[1]<-snps[1,4]/bpcM
sca<-1
pos<-snps[1,4]/bpcM

for (i in 2:length(map)){
    if (snps[i-1,3]!=snps[i,3]) { # new scaffold
		pos<-pos+scafdist
		sca<-sca+1
	}
	pos<-pos+(snps[i,4]-snps[i-1,4])/bpcM
	map[i]<-pos
}
write(signif(map,digits=8),file="$map",ncolumns=length(map))
EOF

open (R, "> recombdist.R") or die "could not write the R file\n";
	print R $recombdist;
close(R);

system "R CMD BATCH recombdist.R\n";
system "rm -f recombdist.R*\n";

## write fastPHASE input file

print OUT "$nind\n";
print OUT "$nloc\n";

open (MAP, $map) or die "Couldn't open SNP positions file $map\n";
$row = <MAP>;
chomp $row;
print OUT "P $row\n";

for $i (1..$nind){
    print OUT "$indids[$i -1]\n";
    system "cut -d ' ' -f\"$i\" \"$tmpa1\" | tr '\n' ' ' > allele1";
    open (TMPA1, "allele1");
    $row = <TMPA1>;
    chomp $row;
    print OUT "$row\n";
    close (TMPA1);
    system "cut -d ' ' -f\"$i\" \"$tmpa2\" | tr '\n' ' ' > allele2";
    open (TMPA2, "allele2");
    $row = <TMPA2>;
    chomp $row;
    print OUT "$row\n";
    close (TMPA2);
}
system "rm -f allele1\n";
system "rm -f allele2\n";

close (OUT);

