#!/usr/bin/env perl

# (c) 2020 - Romain Viloutreix
# romain.villoutreix@gmail.com

# Description: Given a list of bamfiles it return a file with depth coverage along the genome and plots it
# Uses Samptools depth option

use warnings;
use Getopt::Long;
use File::Basename;
use LWP::Simple;
use List::Util qw(sum min);
use Statistics::R;

my $version='1.1--2017-06-14';

&author;
my ($bamlist, $outfile);

#initial values
my $basequal=20;
my $mapqual=20;

#samtools path, version 1.3.1
my $samtools='/usr/local/extras/Genomics/apps/samtools/1.3.1/bin/samtools';

#cumulative length file, reference genome v1.3b2
my $cumlenfile = "/usr/local/extras/Genomics/scripts/gwas/useful_files/tcristinae_draft_1.3b2.scalen.scaf-1.cumlen.dsv";

#regjon of interest
my $interestreg='lg8_ord15_scaf128';


GetOptions(
    'f|F=s' => \$bamlist, # list of input BAM filenames as asked from samtools depth -l option: One name per line 
	'o|O=s' => \$outfile, #name of the output file
	's|S=s' => \$$cumlenfile, # scaf -1 cumlative length file as generated with approriate function: compute_scaf-1_cummulative_length.pl
	'b|B=i' => \$basequal, #base quality threshold
	'q|Q=i' => \$mapqual, #mapping quality threshold
    'r|R=s' => \$interestreg, # a region of interest of the genome as specified by samtools region option (lg8_ord15_scaf128 for scaffold 128)
	'h|help'=> \&usage
) or print ("\nERROR: Option/s not recognized (see above)\n") and &usage;


&usage if (!defined($bamlist));
&usage if (!defined($outfile));

# -----------------------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------------------
#formatting file

print STDERR "\n\n";

print "Necessary files found :) \n\n\  computing coverage depth along selected portion the genome ! \n\n";

print "Base quality threshold = $basequal / Mapping quality threshold = $mapqual \n\n";

my $cmd="$samtools depth -f $bamlist -r $interestreg -q $basequal -Q $mapqual > $outfile\.tmp";

print "  command passed to shell: $cmd\n\n";

system($cmd);

print "  Samtools execution finished. Now processing files with perl!\n\n";

#Opening of the temporary and output files
my @fields=();
my @subfield1=();
my $lg=();
my $scaffold=();
my $scaforder=();
my $position=();
my $line=();
my $sum=();
my $sumind=();
my $genome_pos_w_Ns="NA";

#loading samples names into array for file naming

open my $handle, '<', $bamlist;
chomp(my @samples = <$handle>);
close $handle;

s/\.sorted\.bam//g for @samples;

my $header="lg\tscaforder\tscaffold\tposition\tgenome_pos_w_Ns\t".join("\t",@samples[0..$#samples])."\tnreads\tnindwithreads\n";

open (OFH, ">$outfile\.tmp2") or die "\nCan't open $outfile\.tmp2 file\n\n";
print OFH "$header";

open (IFH, "$outfile\.tmp") or die "\nCan't open $outfile\.tmp file\n\n";
	while (<IFH>){

	chomp $_;
	@fields = split("\t",$_);
	@subfield1 = split("_", $fields[0]);
	$lg = substr $subfield1[0], 2;
	$scaffold = substr $subfield1[2], 4;
	$scaforder = substr $subfield1[1], 3;
	$position = $fields[1];
	
	#compute the sum of reads and number of ind with reads
	$sum=0;
	$sumind=0;
	foreach $s (@fields[2..$#fields]){$sum+=$s;$sumind++ if ($s>0);}

	$line = "$lg\t$scaforder\t$scaffold\t$position\t$genome_pos_w_Ns\t".join("\t",@fields[2..$#fields])."\t$sum\t$sumind\n";
	print OFH "$line";


	}

#closing the files
close(IFH);
close(OFH);

print "  Perl processing finished. Now processing file using R\n\n";

# R code - add real physical position
# # --------------------------------------------------------------------------------------------

my $R = Statistics::R->new();

$R->run(qq`

#### load formatted file with scaffold length
data <- read.table("$outfile\.tmp2", header=TRUE, sep="\t", na.string="NA");

scaflen <- read.table("$cumlenfile", header=TRUE, sep="\t", na.string="NA");

data\$genome_pos_w_Ns <- NA;

for(i in c(unique(data\$scaffold))){
data\$genome_pos_w_Ns[data\$scaffold==i] <- data\$position[data\$scaffold==i] + scaflen\$cumlength_w_Ns[scaflen\$scafnumber==i];
}

write.table(data, file="$outfile", row.names=FALSE, col.names=TRUE, dec=".", na="NA", eol="\n", sep="\t", quote=FALSE);`);

$R->stop();

my $rmcmd1 = "rm $outfile\.tmp";
my $rmcmd2 = "rm $outfile\.tmp2";

system($rmcmd1);
system($rmcmd2);


print "R processing finished. Bye ! \n\n";


# ================================================================================================
# Show copyright
# ==============================================================================
sub author{
    print "\n";
    print "#########################################\n";
    print "  ".basename($0)."\n";
	print "  version $version     \n";
    print "  (c) Romain Villoutreix             \n";
    print "  romain.villoutreix\@gmail.com      \n";
    print "#########################################\n";
	print "\n";
}
# ==============================================================================

# Show usage
# ==============================================================================
sub usage{
    print "\n";
	print "  Usage:\n";
    print "    ".basename($0)."\n";
	print "     !?! WARNING: too demanding for genome wide patterns at the moment / Shall be fixed when time allows !?!\n";
	print "      -f <list of input BAM filenames as asked from samtools depth -l option: One name per line>\n";
	print "      -s <scaffold -1 cummulative length file from compute_scaf-1_cummulative_length.pl function, default=$cumlenfile>\n";
	print "      -o <output file>\n";
	print "      -b <base quality threshold (optional, default=$basequal)>\n";
	print "      -q <mapping quality threshold (optional, default=$mapqual)>\n";
	print "      -r < region of interest (optional, default=$interestreg)>\n";
    print "\n";
    exit;
}
# ==============================================================================
