#!/usr/bin/env perl

# (c) 2020 - Romain Villoutreix
# romain.villoutreix@gmail.com
# Last modified: 15/11/2016

# Description:
# This script plots the depth values (number of reads) resulting from the samtools_pooled_depth.pl function


use warnings;
use strict;
use Getopt::Long;
use File::Basename;
use File::Path qw(make_path);
use File::Spec;
use Statistics::R;
use POSIX qw(strftime);

my $version='1.0.0-2016.11.15';

#Default graphic variables
my $y=0;
my $cumlenfile='/stash/home/bo1revx/romain_scripts/dioscripts/gwas/useful_files/tcristinae_draft_1.3b2.scalen.scaf-1.cumlen.dsv';

#Default outfiles
my $outdir='./';
&author;
my ($infile);
GetOptions( 
    'i|I=s'  => \$infile, 
    'o|O=s'  => \$outdir,
	's|S=s' => \$cumlenfile,
	'y=f' => \$y,
    'h|help' => \&usage 
);

&usage if (!defined($infile) || !defined($outdir));

#Input file does exist ?
if (! -e $infile){
die ("\nCan't access input file: $infile\n\n");
}

# Output directory does exist ?
if (! -e $outdir){
eval {make_path($outdir)}
or die ("\nError creating output directory: $outdir\n\n");
}

#Cumlen file does exist ?
if (! -e $cumlenfile){
die ("\nCan't access scaf-1 cumlenfile : $cumlenfile\n\n");
}
	   


#some specifications of path
$infile=File::Spec->rel2abs($infile);
$outdir=File::Spec->rel2abs($outdir);
my $inbase=basename($infile);
$inbase=~ s/\.txt//g;
my $outbase="$outdir/".basename($infile);
$outbase=~ s/\.txt//g;

print "Necessary files found :) \n\n\  Making graphics of depth along genome ! \n\n"; 

# R code - put whatever you want in here
# --------------------------------------------------------------------------------------------
   my $R = Statistics::R->new();
     
$R->run(qq`

setwd("$outdir");
library(data.table);

#defining graphic variables

	y <- $y;

# opening sparse file and cumlenfile

	depth <- fread("$infile");

	scaflen <- read.table("$cumlenfile", header=TRUE, sep="\t", na.string="NA");

#renamed LGNA into LG14

	depth\$lg[is.na(depth\$lg)] <- 14;
	depth\$scaforder[is.na(depth\$lg)] <- 1;

# plot depth values along the LG

	depth\$colour <- "NA";
	depth\$colour[depth\$lg%%2==0] <- "grey";
	depth\$colour[depth\$lg%%2!=0] <- "blue";

	###taking into account physical map ###
	# prepare useful table for plot

    indextable <- aggregate(depth\$genome_pos_w_Ns, list(depth\$lg), mean);

	jpeg(filename="$outbase\_depth_along_LG_w_phy_pos.jpg", width=21, height=10, units="cm", pointsize=12, res=600);

    if(y==0){
		
		 plot(NULL, NULL, xlab = "linkage group", ylab = "nreads", ylim = c(0, max(depth\$nreads, na.rm=TRUE)),  xlim = c(0,(max(scaflen\$cumlength_w_Ns)+tail(scaflen\$length_w_Ns, 1))), bty = "n", xaxt = "n");
		 points(depth\$genome_pos_w_Ns, depth\$nreads, col = c(depth\$colour), cex = 0.35, pch=16);
		 text(x=c(indextable[,2]), y = rep((0-(max(depth\$nreads, na.rm=TRUE)-0)*0.10), nrow(indextable)), labels = c(indextable[,1]), xpd=TRUE, cex=0.8);
		
	}

    if(y!=0){
		
		plot(NULL, NULL, xlab = "linkage group", ylab = "nreads", ylim = c(0, y2),  xlim = c(0,(max(scaflen\$cumlength_w_Ns)+tail(scaflen\$length_w_Ns, 1))), bty = "n", xaxt = "n");
		points(depth\$genome_pos_w_Ns, depth\$nreads, col = c(depth\$colour), cex = 0.35, pch=16);
		text(x=c(indextable[,2]), y = rep((0-(y2-0)*0.10), nrow(indextable)), labels = c(indextable[,1]), xpd=TRUE, cex=0.8);
		
	}


	dev.off();`);

$R->stop();

print " Done!  Graphs saved in $outdir \n\n  Bye \n\n";

# --------------------------------------------------------------------------------------------#


# ==============================================================================
# ==============================================================================
# ============================== SUBROUTINES ===================================
# ==============================================================================
# ==============================================================================

# Show copyright
# ==============================================================================
sub author{
    print "\n";
    print "#########################################\n";
    print "  ".basename($0)."\n";
	print "  version $version\n";
    print "  (c) Romain Villoutreix            \n";
    print "  romain.villoutreix\@gmail.com      \n";
    print "#########################################\n";
	print "\n";
}
# ==============================================================================

# Show usage
# ==============================================================================
sub usage{
    print "\n";
	print "  Usage:\n";
    print "    ".basename($0)."\n";
	print "      -i <input file (output from samtools_pooled_depth.pl)>\n";
	print "      -o <output directory - optional defaults to $outdir>\n";
	print "      -s <scaffold -1 cummulative length file from compute_scaf-1_cummulative_length.pl function, default v1.3b2 of the genome>\n";
	print "      -y <max y display value for depth along LG graphic / default = automatic value>\n";
	print "      -h <show this help>\n";
    print "\n";
    exit;
}
# ==============================================================================
