#!/usr/bin/env perl

# (c) 2020 - Romain Villoutreix
# romain.villoutreix@gmail.com
# Last modified: 15/06/2017

# Description:
# This script plots the depth values (number of reads) resulting from the samtools_pooled_depth.pl function


use warnings;
use strict;
use Getopt::Long;
use File::Basename;
use File::Path qw(make_path);
use File::Spec;
use Statistics::R;
use POSIX qw(strftime);

my $version='1.0.0-2017.06.15';

#Default graphic variables
my $y=0;

#Default outfiles
my $outdir='./';

#defaults interest scaffold, start and end
my $interestscaf=128;
my $start=5000000;
my $end=6000000;

&author;
my ($infile);
GetOptions( 
    'i|I=s'  => \$infile, 
    'o|O=s'  => \$outdir,
	's|S=f' => \$interestscaf,
	'start=i' => \$start,
	'end=i' => \$end,
	'y=f' => \$y,
    'h|help' => \&usage 
);

&usage if (!defined($infile) || !defined($outdir));

#Input file does exist ?
if (! -e $infile){
die ("\nCan't access input file: $infile\n\n");
}

# Output directory does exist ?
if (! -e $outdir){
eval {make_path($outdir)}
or die ("\nError creating output directory: $outdir\n\n");
}

#some specifications of path
$infile=File::Spec->rel2abs($infile);
$outdir=File::Spec->rel2abs($outdir);
my $inbase=basename($infile);
$inbase=~ s/\.txt//g;
my $outbase="$outdir/".basename($infile);
$outbase=~ s/\.txt//g;

print "Necessary files found :) \n\n";

print "\tdepth infile used: $infile\n\n";

print "\tMaking zoomed graphics of depth along scaffold $interestscaf -  start: $start bp - end $end bp\n\n"; 

print "\tstarting R\n\n";

# R code - put whatever you want in here
# --------------------------------------------------------------------------------------------
   my $R = Statistics::R->new();
     
$R->run(qq`

setwd("$outdir");
library(data.table);

#defining graphic variables

	y <- $y;

# opening sparse file

	depth <- as.data.table(fread("$infile", sep="\\t", dec=".", header=TRUE));

#renamed LGNA into LG14

	depth\$lg[is.na(depth\$lg)] <- 14;
	depth\$scaforder[is.na(depth\$lg)] <- 1;

# plot depth values along interest scaffold

	results=depth[scaffold==$interestscaf,];
	results=results[position>=$start,];
	results=results[position<=$end,];
	results[, \`:=\`(col="grey40")];

	jpeg(filename="$outbase\_depth_w_phy_pos_along_scaf$interestscaf\_start$start\_end$end\_ylim$y\.jpg", width=21, height=10, units="cm", pointsize=12, res=600);

    if(y==0){
		
		 plot(NULL, NULL, xlab = "scaffold $interestscaf", ylab = "nreads", ylim = c(0, max(results\$nreads, na.rm=TRUE)),  xlim = c($start, $end), bty = "n", xaxt = "n");
		 points(results\$position, results\$nreads, col = c(results\$col), cex = 0.35, pch=16);
		 axis(1, seq(0, max(results[,position]), 500000));
		
	}

    if(y!=0){
		
		plot(NULL, NULL, xlab = "scaffold $interestscaf", ylab = "nreads", ylim = c(0, y),  xlim = c($start, $end), bty = "n", xaxt = "n");
		points(results\$position, results\$nreads, col = c(results\$col), cex = 0.35, pch=16);
		axis(1, seq(0, max(results[,position]), 500000));
		
	}


	dev.off();`);

$R->stop();

print " Done!  Graphs saved in $outdir \n\n  Bye \n\n";

# --------------------------------------------------------------------------------------------#


# ==============================================================================
# ==============================================================================
# ============================== SUBROUTINES ===================================
# ==============================================================================
# ==============================================================================

# Show copyright
# ==============================================================================
sub author{
    print "\n";
    print "#########################################\n";
    print "  ".basename($0)."\n";
	print "  version $version\n";
    print "  (c) Romain Villoutreix            \n";
    print "  romain.villoutreix\@gmail.com      \n";
    print "#########################################\n";
	print "\n";
}
# ==============================================================================

# Show usage
# ==============================================================================
sub usage{
    print "\n";
	print "  Usage:\n";
    print "    ".basename($0)."\n";
	print "      -i <input file (output from samtools_pooled_depth.pl)>\n";
	print "      -o <output directory - optional defaults=$outdir>\n";
	print "      -s <interest scaffold - optional defaults=$interestscaf>\n";
	print "      -start <start position, bp from beginning of scaffold - optional defaults=$start>\n";
	print "      -end <end position, bp from beginning of scaffold - optional defaults=$end>\n";
	print "      -y <max y display value for depth along LG graphic / default=automatic value>\n";
	print "      -h <show this help>\n";
    print "\n";
    exit;
}
# ==============================================================================
